import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.pension_rule import PensionRule
from ..models.tiered_pension_rate import TieredPensionRate
from ..models.worker_group import WorkerGroup
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionSummary")


@attr.s(auto_attribs=True)
class PensionSummary:
    """If a PayRunEntry contains pension contributions then it'll also include a PensionSummary model
    giving further information about the Pension Scheme and the contributions made

        Attributes:
            pension_id (Union[Unset, str]): [readonly] The Id of the Pension.
            name (Union[Unset, None, str]): [readonly] The name of the PensionScheme to which contributions have been made.
            pension_scheme_id (Union[Unset, str]): [readonly] The Id of the PensionScheme.
            start_date (Union[Unset, datetime.date]): [readonly]
            worker_group_id (Union[Unset, str]): [readonly] The Id of the WorkerGroup.
            pension_rule (Union[Unset, PensionRule]):
            papdis_pension_provider_id (Union[Unset, None, str]): [readonly] Papdis information from the PensionScheme
            papdis_employer_id (Union[Unset, None, str]): [readonly] Papdis information from the PensionScheme
            employee_pension_contribution_multiplier (Union[Unset, float]): [readonly] If the PensionScheme is set to
                SubtractBasicRateTax then this value  is used to reduce the contribution amount.
                Otherwise it is set as 1.
            additional_voluntary_contribution (Union[Unset, float]): [readonly] Any Additional Voluntary Contribution the
                Employee has chosen to make
                Otherwise it is set as 1.
            avc_is_percentage (Union[Unset, bool]): [readonly] Determines whether the Value of the Additional Voluntary
                Contribution is a fixed amount or a percentage,
            auto_enrolled (Union[Unset, bool]): [readonly] Any Additional Voluntary Contribution the Employee has chosen to
                make
                Otherwise it is set as 1.
            worker_group (Union[Unset, WorkerGroup]):
            forced_tier (Union[Unset, None, str]): [readonly] If the WorkerGroup ContributionLevelType is a Tiered Scheme
                then the name of the tier to force the employee on to may be specified.
                If none is specified then the Tier is determined by the earnings in the period
            tiers (Union[Unset, None, List[TieredPensionRate]]):
            assumed_pensionable_pay (Union[Unset, None, float]): [readonly] Assumed Pensionable Pay. If the employee is
                receiving any Statutory Leave that has an AssumedPensionablePay value set
                then it'll be shown here.
            pensionable_pay_codes (Union[Unset, None, List[str]]): [readonly] If the pension scheme is set to override the
                Pensionale PayCodes, then this is what they've been set to.
    """

    pension_id: Union[Unset, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    pension_scheme_id: Union[Unset, str] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    worker_group_id: Union[Unset, str] = UNSET
    pension_rule: Union[Unset, PensionRule] = UNSET
    papdis_pension_provider_id: Union[Unset, None, str] = UNSET
    papdis_employer_id: Union[Unset, None, str] = UNSET
    employee_pension_contribution_multiplier: Union[Unset, float] = UNSET
    additional_voluntary_contribution: Union[Unset, float] = UNSET
    avc_is_percentage: Union[Unset, bool] = UNSET
    auto_enrolled: Union[Unset, bool] = UNSET
    worker_group: Union[Unset, WorkerGroup] = UNSET
    forced_tier: Union[Unset, None, str] = UNSET
    tiers: Union[Unset, None, List[TieredPensionRate]] = UNSET
    assumed_pensionable_pay: Union[Unset, None, float] = UNSET
    pensionable_pay_codes: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pension_id = self.pension_id
        name = self.name
        pension_scheme_id = self.pension_scheme_id
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        worker_group_id = self.worker_group_id
        pension_rule: Union[Unset, str] = UNSET
        if not isinstance(self.pension_rule, Unset):
            pension_rule = self.pension_rule.value

        papdis_pension_provider_id = self.papdis_pension_provider_id
        papdis_employer_id = self.papdis_employer_id
        employee_pension_contribution_multiplier = (
            self.employee_pension_contribution_multiplier
        )
        additional_voluntary_contribution = self.additional_voluntary_contribution
        avc_is_percentage = self.avc_is_percentage
        auto_enrolled = self.auto_enrolled
        worker_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.worker_group, Unset):
            worker_group = self.worker_group.to_dict()

        forced_tier = self.forced_tier
        tiers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tiers, Unset):
            if self.tiers is None:
                tiers = None
            else:
                tiers = []
                for tiers_item_data in self.tiers:
                    tiers_item = tiers_item_data.to_dict()

                    tiers.append(tiers_item)

        assumed_pensionable_pay = self.assumed_pensionable_pay
        pensionable_pay_codes: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.pensionable_pay_codes, Unset):
            if self.pensionable_pay_codes is None:
                pensionable_pay_codes = None
            else:
                pensionable_pay_codes = self.pensionable_pay_codes

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pension_id is not UNSET:
            field_dict["pensionId"] = pension_id
        if name is not UNSET:
            field_dict["name"] = name
        if pension_scheme_id is not UNSET:
            field_dict["pensionSchemeId"] = pension_scheme_id
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if worker_group_id is not UNSET:
            field_dict["workerGroupId"] = worker_group_id
        if pension_rule is not UNSET:
            field_dict["pensionRule"] = pension_rule
        if papdis_pension_provider_id is not UNSET:
            field_dict["papdisPensionProviderId"] = papdis_pension_provider_id
        if papdis_employer_id is not UNSET:
            field_dict["papdisEmployerId"] = papdis_employer_id
        if employee_pension_contribution_multiplier is not UNSET:
            field_dict[
                "employeePensionContributionMultiplier"
            ] = employee_pension_contribution_multiplier
        if additional_voluntary_contribution is not UNSET:
            field_dict[
                "additionalVoluntaryContribution"
            ] = additional_voluntary_contribution
        if avc_is_percentage is not UNSET:
            field_dict["avcIsPercentage"] = avc_is_percentage
        if auto_enrolled is not UNSET:
            field_dict["autoEnrolled"] = auto_enrolled
        if worker_group is not UNSET:
            field_dict["workerGroup"] = worker_group
        if forced_tier is not UNSET:
            field_dict["forcedTier"] = forced_tier
        if tiers is not UNSET:
            field_dict["tiers"] = tiers
        if assumed_pensionable_pay is not UNSET:
            field_dict["assumedPensionablePay"] = assumed_pensionable_pay
        if pensionable_pay_codes is not UNSET:
            field_dict["pensionablePayCodes"] = pensionable_pay_codes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pension_id = d.pop("pensionId", UNSET)

        name = d.pop("name", UNSET)

        pension_scheme_id = d.pop("pensionSchemeId", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        worker_group_id = d.pop("workerGroupId", UNSET)

        _pension_rule = d.pop("pensionRule", UNSET)
        pension_rule: Union[Unset, PensionRule]
        if isinstance(_pension_rule, Unset):
            pension_rule = UNSET
        else:
            pension_rule = PensionRule(_pension_rule)

        papdis_pension_provider_id = d.pop("papdisPensionProviderId", UNSET)

        papdis_employer_id = d.pop("papdisEmployerId", UNSET)

        employee_pension_contribution_multiplier = d.pop(
            "employeePensionContributionMultiplier", UNSET
        )

        additional_voluntary_contribution = d.pop(
            "additionalVoluntaryContribution", UNSET
        )

        avc_is_percentage = d.pop("avcIsPercentage", UNSET)

        auto_enrolled = d.pop("autoEnrolled", UNSET)

        _worker_group = d.pop("workerGroup", UNSET)
        worker_group: Union[Unset, WorkerGroup]
        if isinstance(_worker_group, Unset):
            worker_group = UNSET
        else:
            worker_group = WorkerGroup.from_dict(_worker_group)

        forced_tier = d.pop("forcedTier", UNSET)

        tiers = []
        _tiers = d.pop("tiers", UNSET)
        for tiers_item_data in _tiers or []:
            tiers_item = TieredPensionRate.from_dict(tiers_item_data)

            tiers.append(tiers_item)

        assumed_pensionable_pay = d.pop("assumedPensionablePay", UNSET)

        pensionable_pay_codes = cast(List[str], d.pop("pensionablePayCodes", UNSET))

        pension_summary = cls(
            pension_id=pension_id,
            name=name,
            pension_scheme_id=pension_scheme_id,
            start_date=start_date,
            worker_group_id=worker_group_id,
            pension_rule=pension_rule,
            papdis_pension_provider_id=papdis_pension_provider_id,
            papdis_employer_id=papdis_employer_id,
            employee_pension_contribution_multiplier=employee_pension_contribution_multiplier,
            additional_voluntary_contribution=additional_voluntary_contribution,
            avc_is_percentage=avc_is_percentage,
            auto_enrolled=auto_enrolled,
            worker_group=worker_group,
            forced_tier=forced_tier,
            tiers=tiers,
            assumed_pensionable_pay=assumed_pensionable_pay,
            pensionable_pay_codes=pensionable_pay_codes,
        )

        return pension_summary



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.update_photo_account_multipart_data import (
    UpdatePhotoAccountMultipartData,
)
from ...models.user import User
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    multipart_data: UpdatePhotoAccountMultipartData,
) -> Dict[str, Any]:
    url = "{}/account/photo".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
    }


def _parse_response(*, response: httpx.Response) -> Optional[User]:
    if response.status_code == 200:
        response_200 = User.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[User]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    multipart_data: UpdatePhotoAccountMultipartData,
) -> Response[User]:
    """Update Photo

     Submit an image here and we'll upload it, resize it to 200px squared and set it as the image for
    your account.

    Args:
        multipart_data (UpdatePhotoAccountMultipartData):

    Returns:
        Response[User]
    """

    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    multipart_data: UpdatePhotoAccountMultipartData,
) -> Optional[User]:
    """Update Photo

     Submit an image here and we'll upload it, resize it to 200px squared and set it as the image for
    your account.

    Args:
        multipart_data (UpdatePhotoAccountMultipartData):

    Returns:
        Response[User]
    """

    return sync_detailed(
        client=client,
        multipart_data=multipart_data,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    multipart_data: UpdatePhotoAccountMultipartData,
) -> Response[User]:
    """Update Photo

     Submit an image here and we'll upload it, resize it to 200px squared and set it as the image for
    your account.

    Args:
        multipart_data (UpdatePhotoAccountMultipartData):

    Returns:
        Response[User]
    """

    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    multipart_data: UpdatePhotoAccountMultipartData,
) -> Optional[User]:
    """Update Photo

     Submit an image here and we'll upload it, resize it to 200px squared and set it as the image for
    your account.

    Args:
        multipart_data (UpdatePhotoAccountMultipartData):

    Returns:
        Response[User]
    """

    return (
        await asyncio_detailed(
            client=client,
            multipart_data=multipart_data,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CookiePreference")


@attr.s(auto_attribs=True)
class CookiePreference:
    """
    Attributes:
        gainsight (Union[Unset, None, bool]):
    """

    gainsight: Union[Unset, None, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        gainsight = self.gainsight

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if gainsight is not UNSET:
            field_dict["gainsight"] = gainsight

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        gainsight = d.pop("gainsight", UNSET)

        cookie_preference = cls(
            gainsight=gainsight,
        )

        return cookie_preference



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.linked_piw_result import LinkedPiwResult
from ..types import UNSET, Unset

T = TypeVar("T", bound="LinkedPiw")


@attr.s(auto_attribs=True)
class LinkedPiw:
    """Linked Period of Incapacity for Work.
    If you record Sick Leave and select Statutory Pay then any other Sick Leave with Statutory Pay
    lasting 4 or more days in the previous 8 weeks will be linked to it

        Attributes:
            result (Union[Unset, LinkedPiwResult]):
            id (Union[Unset, str]): [readonly] The Id of the linked Leave
            average_weekly_earnings (Union[Unset, float]): [readonly] Average weekly earnings from linked Leave
            working_days (Union[Unset, float]): [readonly] The number of working days in the linked Leave.
            total_days (Union[Unset, float]): [readonly] The number of days covered by the linked Leave.
            ssp_first_pay_day (Union[Unset, None, datetime.date]): [readonly] The SspFirstPayDay from the linked Leave, if
                set
            ssp_first_day (Union[Unset, None, datetime.date]): [readonly] The SspFirstDay from the linked Leave, if set
    """

    result: Union[Unset, LinkedPiwResult] = UNSET
    id: Union[Unset, str] = UNSET
    average_weekly_earnings: Union[Unset, float] = UNSET
    working_days: Union[Unset, float] = UNSET
    total_days: Union[Unset, float] = UNSET
    ssp_first_pay_day: Union[Unset, None, datetime.date] = UNSET
    ssp_first_day: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        result: Union[Unset, str] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.value

        id = self.id
        average_weekly_earnings = self.average_weekly_earnings
        working_days = self.working_days
        total_days = self.total_days
        ssp_first_pay_day: Union[Unset, None, str] = UNSET
        if not isinstance(self.ssp_first_pay_day, Unset):
            ssp_first_pay_day = (
                self.ssp_first_pay_day.isoformat() if self.ssp_first_pay_day else None
            )

        ssp_first_day: Union[Unset, None, str] = UNSET
        if not isinstance(self.ssp_first_day, Unset):
            ssp_first_day = (
                self.ssp_first_day.isoformat() if self.ssp_first_day else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if result is not UNSET:
            field_dict["result"] = result
        if id is not UNSET:
            field_dict["id"] = id
        if average_weekly_earnings is not UNSET:
            field_dict["averageWeeklyEarnings"] = average_weekly_earnings
        if working_days is not UNSET:
            field_dict["workingDays"] = working_days
        if total_days is not UNSET:
            field_dict["totalDays"] = total_days
        if ssp_first_pay_day is not UNSET:
            field_dict["sspFirstPayDay"] = ssp_first_pay_day
        if ssp_first_day is not UNSET:
            field_dict["sspFirstDay"] = ssp_first_day

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _result = d.pop("result", UNSET)
        result: Union[Unset, LinkedPiwResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = LinkedPiwResult(_result)

        id = d.pop("id", UNSET)

        average_weekly_earnings = d.pop("averageWeeklyEarnings", UNSET)

        working_days = d.pop("workingDays", UNSET)

        total_days = d.pop("totalDays", UNSET)

        _ssp_first_pay_day = d.pop("sspFirstPayDay", UNSET)
        ssp_first_pay_day: Union[Unset, None, datetime.date]
        if _ssp_first_pay_day is None:
            ssp_first_pay_day = None
        elif isinstance(_ssp_first_pay_day, Unset):
            ssp_first_pay_day = UNSET
        else:
            ssp_first_pay_day = isoparse(_ssp_first_pay_day).date()

        _ssp_first_day = d.pop("sspFirstDay", UNSET)
        ssp_first_day: Union[Unset, None, datetime.date]
        if _ssp_first_day is None:
            ssp_first_day = None
        elif isinstance(_ssp_first_day, Unset):
            ssp_first_day = UNSET
        else:
            ssp_first_day = isoparse(_ssp_first_day).date()

        linked_piw = cls(
            result=result,
            id=id,
            average_weekly_earnings=average_weekly_earnings,
            working_days=working_days,
            total_days=total_days,
            ssp_first_pay_day=ssp_first_pay_day,
            ssp_first_day=ssp_first_day,
        )

        return linked_piw



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.right_to_work_document_type import RightToWorkDocumentType
from ..types import UNSET, Unset

T = TypeVar("T", bound="RightToWork")


@attr.s(auto_attribs=True)
class RightToWork:
    """
    Attributes:
        checked (Union[Unset, bool]):
        document_type (Union[Unset, RightToWorkDocumentType]):
        document_ref (Union[Unset, None, str]):
        document_expiry (Union[Unset, None, datetime.date]):
        note (Union[Unset, None, str]):
    """

    checked: Union[Unset, bool] = UNSET
    document_type: Union[Unset, RightToWorkDocumentType] = UNSET
    document_ref: Union[Unset, None, str] = UNSET
    document_expiry: Union[Unset, None, datetime.date] = UNSET
    note: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        checked = self.checked
        document_type: Union[Unset, str] = UNSET
        if not isinstance(self.document_type, Unset):
            document_type = self.document_type.value

        document_ref = self.document_ref
        document_expiry: Union[Unset, None, str] = UNSET
        if not isinstance(self.document_expiry, Unset):
            document_expiry = (
                self.document_expiry.isoformat() if self.document_expiry else None
            )

        note = self.note

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if checked is not UNSET:
            field_dict["checked"] = checked
        if document_type is not UNSET:
            field_dict["documentType"] = document_type
        if document_ref is not UNSET:
            field_dict["documentRef"] = document_ref
        if document_expiry is not UNSET:
            field_dict["documentExpiry"] = document_expiry
        if note is not UNSET:
            field_dict["note"] = note

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        checked = d.pop("checked", UNSET)

        _document_type = d.pop("documentType", UNSET)
        document_type: Union[Unset, RightToWorkDocumentType]
        if isinstance(_document_type, Unset):
            document_type = UNSET
        else:
            document_type = RightToWorkDocumentType(_document_type)

        document_ref = d.pop("documentRef", UNSET)

        _document_expiry = d.pop("documentExpiry", UNSET)
        document_expiry: Union[Unset, None, datetime.date]
        if _document_expiry is None:
            document_expiry = None
        elif isinstance(_document_expiry, Unset):
            document_expiry = UNSET
        else:
            document_expiry = isoparse(_document_expiry).date()

        note = d.pop("note", UNSET)

        right_to_work = cls(
            checked=checked,
            document_type=document_type,
            document_ref=document_ref,
            document_expiry=document_expiry,
            note=note,
        )

        return right_to_work



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="Post")


@attr.s(auto_attribs=True)
class Post:
    """
    Attributes:
        title (Union[Unset, None, str]): Title of the Post
        is_primary (Union[Unset, bool]): Set to True if this is Primary post of the Employee
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    title: Union[Unset, None, str] = UNSET
    is_primary: Union[Unset, bool] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        is_primary = self.is_primary
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if is_primary is not UNSET:
            field_dict["isPrimary"] = is_primary
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title", UNSET)

        is_primary = d.pop("isPrimary", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        post = cls(
            title=title,
            is_primary=is_primary,
            employee=employee,
            id=id,
        )

        return post



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.employee import Employee
from ..models.external_data_provider_id import ExternalDataProviderId
from ..models.external_employee_mapping_status import ExternalEmployeeMappingStatus
from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExternalEmployeeMapping")


@attr.s(auto_attribs=True)
class ExternalEmployeeMapping:
    """Used to represent details of an employee from an ExternalDataProvider, along with mapping information to an employee
    in the payroll system

        Attributes:
            external_id (Union[Unset, None, str]): [readonly] The id for the employee in the external system
            provider_id (Union[Unset, ExternalDataProviderId]):
            status (Union[Unset, ExternalEmployeeMappingStatus]):
            employee (Union[Unset, Item]):
            external_employee (Union[Unset, Employee]):
            last_sync_date (Union[Unset, None, datetime.date]): [readonly] The date and time this mapping was last
                synchronised
    """

    external_id: Union[Unset, None, str] = UNSET
    provider_id: Union[Unset, ExternalDataProviderId] = UNSET
    status: Union[Unset, ExternalEmployeeMappingStatus] = UNSET
    employee: Union[Unset, Item] = UNSET
    external_employee: Union[Unset, Employee] = UNSET
    last_sync_date: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        external_id = self.external_id
        provider_id: Union[Unset, str] = UNSET
        if not isinstance(self.provider_id, Unset):
            provider_id = self.provider_id.value

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        external_employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.external_employee, Unset):
            external_employee = self.external_employee.to_dict()

        last_sync_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_sync_date, Unset):
            last_sync_date = (
                self.last_sync_date.isoformat() if self.last_sync_date else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if provider_id is not UNSET:
            field_dict["providerId"] = provider_id
        if status is not UNSET:
            field_dict["status"] = status
        if employee is not UNSET:
            field_dict["employee"] = employee
        if external_employee is not UNSET:
            field_dict["externalEmployee"] = external_employee
        if last_sync_date is not UNSET:
            field_dict["lastSyncDate"] = last_sync_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        external_id = d.pop("externalId", UNSET)

        _provider_id = d.pop("providerId", UNSET)
        provider_id: Union[Unset, ExternalDataProviderId]
        if isinstance(_provider_id, Unset):
            provider_id = UNSET
        else:
            provider_id = ExternalDataProviderId(_provider_id)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ExternalEmployeeMappingStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ExternalEmployeeMappingStatus(_status)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        _external_employee = d.pop("externalEmployee", UNSET)
        external_employee: Union[Unset, Employee]
        if isinstance(_external_employee, Unset):
            external_employee = UNSET
        else:
            external_employee = Employee.from_dict(_external_employee)

        _last_sync_date = d.pop("lastSyncDate", UNSET)
        last_sync_date: Union[Unset, None, datetime.date]
        if _last_sync_date is None:
            last_sync_date = None
        elif isinstance(_last_sync_date, Unset):
            last_sync_date = UNSET
        else:
            last_sync_date = isoparse(_last_sync_date).date()

        external_employee_mapping = cls(
            external_id=external_id,
            provider_id=provider_id,
            status=status,
            employee=employee,
            external_employee=external_employee,
            last_sync_date=last_sync_date,
        )

        return external_employee_mapping



from typing import Any, Dict, Union

import httpx

from ...client import Client
from ...models.role import Role
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    user_id: str,
    *,
    client: Client,
    role: Union[Unset, None, Role] = UNSET,
) -> Dict[str, Any]:
    url = "{}/tenants/{id}/users/{userId}/omnipresentrole".format(
        client.base_url, id=id, userId=user_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_role: Union[Unset, None, str] = UNSET
    if not isinstance(role, Unset):
        json_role = role.value if role else None

    params["role"] = json_role

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    id: str,
    user_id: str,
    *,
    client: Client,
    role: Union[Unset, None, Role] = UNSET,
) -> Response[Any]:
    """Set Omnipresent Role

     If the Tenant has EnableOmnipresentUsers enabled then they have the ability to make users have
    automatic access to all employers on that tenant with the specified Role.
    This end point updates the Omnipresent Role of the user.

    Args:
        id (str):
        user_id (str):
        role (Union[Unset, None, Role]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        user_id=user_id,
        client=client,
        role=role,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    id: str,
    user_id: str,
    *,
    client: Client,
    role: Union[Unset, None, Role] = UNSET,
) -> Response[Any]:
    """Set Omnipresent Role

     If the Tenant has EnableOmnipresentUsers enabled then they have the ability to make users have
    automatic access to all employers on that tenant with the specified Role.
    This end point updates the Omnipresent Role of the user.

    Args:
        id (str):
        user_id (str):
        role (Union[Unset, None, Role]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        user_id=user_id,
        client=client,
        role=role,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.holiday_type import HolidayType
from ..types import UNSET, Unset

T = TypeVar("T", bound="LeaveSettings")


@attr.s(auto_attribs=True)
class LeaveSettings:
    """
    Attributes:
        use_default_holiday_type (Union[Unset, bool]): If true then the value for HolidayType comes from the Employer
            record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_allowance_reset_date (Union[Unset, bool]): If true then the value for the AllowanceResetDate comes
            from the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_allowance (Union[Unset, bool]): If true then the value for the Allowance comes from the Employer
            record.
            This property only appears if the LeaveSettings if a child of an Employee (not of an Employer)
        use_default_accrue_payment_in_lieu (Union[Unset, bool]): If true then the value for AccruePaymentInLieu comes
            from the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_accrue_payment_in_lieu_rate (Union[Unset, bool]): If true then the value for AccruePaymentInLieuRate
            comes from the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_accrue_payment_in_lieu_all_gross_pay (Union[Unset, bool]): If true then the value for
            AccruePaymentInLieuAllGrossPay comes from the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_accrue_payment_in_lieu_pay_automatically (Union[Unset, bool]): If true then the value for
            AccruePaymentInLieu comes from the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        use_default_accrue_hours_per_day (Union[Unset, bool]): If true then the value for AccrueHoursPerDay comes from
            the Employer record.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        allowance_reset_date (Union[Unset, datetime.date]): The date that the holiday allowance resets. Only the
            day/month part of the value is relevant.
        allowance (Union[Unset, float]): The number of days holiday an employee can take per year if HolidayType is
            Days.
            Otherwise this is readonly and gives you the number of days accrued since the last reset
        adjustment (Union[Unset, None, float]): Adjustment to number of hours/days/weeks holiday this employee can take
            per year.
            Will reset to 0 when the Allowance resets.
            This property only appears if the LeaveSettings is a child of an Employee (not of an Employer)
        allowance_used (Union[Unset, float]): [readonly] The number of days used from the allowance since last reset
        allowance_used_previous_period (Union[Unset, float]): [readonly] The number of days used in the 12 months prior
            to the last reset
        allowance_remaining (Union[Unset, float]): [readonly] The number of days remaining of the allowance until next
            reset
        holiday_type (Union[Unset, HolidayType]):
        accrue_set_amount (Union[Unset, bool]): If true and HolidayType is Accrual_Days then the AccruePaymentInLieuRate
            will be treated as the set amount to accrue per period worked.
        accrue_hours_per_day (Union[Unset, float]): If HolidayType is Accrual_Days then this value is used to help
            convert hours worked into days accrued
        show_allowance_on_payslip (Union[Unset, bool]): If true then the remaining Allowance will be shown on the
            employees payslip.
        show_ahp_on_payslip (Union[Unset, bool]): If true then the AHP balance will be shown on the employees payslip.
        accrue_payment_in_lieu_rate (Union[Unset, float]): The rate at which Payments in Lieu acrrue. Typically this
            should be 12.07%.
        accrue_payment_in_lieu_all_gross_pay (Union[Unset, bool]): Set to true if you want accrued holiday payments to
            be calculated on the total gross pay for the employee or just on the single regular pay element
        accrue_payment_in_lieu_pay_automatically (Union[Unset, bool]): Set to true if you want employees to be
            automatically paid any outstanding holiday pay
        accrued_payment_liability (Union[Unset, float]): [readonly] The total accrued payments for this employee over
            the lifetime of their employment so far
        accrued_payment_adjustment (Union[Unset, float]): Any manual adjustment to the total accrued
        accrued_payment_paid (Union[Unset, float]): [readonly] The Total amount paid to this employee in lieu of
            holidays
        accrued_payment_balance (Union[Unset, float]): [readonly] The balance of what is owed to this employee in lieu
            of holidays
    """

    use_default_holiday_type: Union[Unset, bool] = UNSET
    use_default_allowance_reset_date: Union[Unset, bool] = UNSET
    use_default_allowance: Union[Unset, bool] = UNSET
    use_default_accrue_payment_in_lieu: Union[Unset, bool] = UNSET
    use_default_accrue_payment_in_lieu_rate: Union[Unset, bool] = UNSET
    use_default_accrue_payment_in_lieu_all_gross_pay: Union[Unset, bool] = UNSET
    use_default_accrue_payment_in_lieu_pay_automatically: Union[Unset, bool] = UNSET
    use_default_accrue_hours_per_day: Union[Unset, bool] = UNSET
    allowance_reset_date: Union[Unset, datetime.date] = UNSET
    allowance: Union[Unset, float] = UNSET
    adjustment: Union[Unset, None, float] = UNSET
    allowance_used: Union[Unset, float] = UNSET
    allowance_used_previous_period: Union[Unset, float] = UNSET
    allowance_remaining: Union[Unset, float] = UNSET
    holiday_type: Union[Unset, HolidayType] = UNSET
    accrue_set_amount: Union[Unset, bool] = UNSET
    accrue_hours_per_day: Union[Unset, float] = UNSET
    show_allowance_on_payslip: Union[Unset, bool] = UNSET
    show_ahp_on_payslip: Union[Unset, bool] = UNSET
    accrue_payment_in_lieu_rate: Union[Unset, float] = UNSET
    accrue_payment_in_lieu_all_gross_pay: Union[Unset, bool] = UNSET
    accrue_payment_in_lieu_pay_automatically: Union[Unset, bool] = UNSET
    accrued_payment_liability: Union[Unset, float] = UNSET
    accrued_payment_adjustment: Union[Unset, float] = UNSET
    accrued_payment_paid: Union[Unset, float] = UNSET
    accrued_payment_balance: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        use_default_holiday_type = self.use_default_holiday_type
        use_default_allowance_reset_date = self.use_default_allowance_reset_date
        use_default_allowance = self.use_default_allowance
        use_default_accrue_payment_in_lieu = self.use_default_accrue_payment_in_lieu
        use_default_accrue_payment_in_lieu_rate = (
            self.use_default_accrue_payment_in_lieu_rate
        )
        use_default_accrue_payment_in_lieu_all_gross_pay = (
            self.use_default_accrue_payment_in_lieu_all_gross_pay
        )
        use_default_accrue_payment_in_lieu_pay_automatically = (
            self.use_default_accrue_payment_in_lieu_pay_automatically
        )
        use_default_accrue_hours_per_day = self.use_default_accrue_hours_per_day
        allowance_reset_date: Union[Unset, str] = UNSET
        if not isinstance(self.allowance_reset_date, Unset):
            allowance_reset_date = self.allowance_reset_date.isoformat()

        allowance = self.allowance
        adjustment = self.adjustment
        allowance_used = self.allowance_used
        allowance_used_previous_period = self.allowance_used_previous_period
        allowance_remaining = self.allowance_remaining
        holiday_type: Union[Unset, str] = UNSET
        if not isinstance(self.holiday_type, Unset):
            holiday_type = self.holiday_type.value

        accrue_set_amount = self.accrue_set_amount
        accrue_hours_per_day = self.accrue_hours_per_day
        show_allowance_on_payslip = self.show_allowance_on_payslip
        show_ahp_on_payslip = self.show_ahp_on_payslip
        accrue_payment_in_lieu_rate = self.accrue_payment_in_lieu_rate
        accrue_payment_in_lieu_all_gross_pay = self.accrue_payment_in_lieu_all_gross_pay
        accrue_payment_in_lieu_pay_automatically = (
            self.accrue_payment_in_lieu_pay_automatically
        )
        accrued_payment_liability = self.accrued_payment_liability
        accrued_payment_adjustment = self.accrued_payment_adjustment
        accrued_payment_paid = self.accrued_payment_paid
        accrued_payment_balance = self.accrued_payment_balance

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if use_default_holiday_type is not UNSET:
            field_dict["useDefaultHolidayType"] = use_default_holiday_type
        if use_default_allowance_reset_date is not UNSET:
            field_dict[
                "useDefaultAllowanceResetDate"
            ] = use_default_allowance_reset_date
        if use_default_allowance is not UNSET:
            field_dict["useDefaultAllowance"] = use_default_allowance
        if use_default_accrue_payment_in_lieu is not UNSET:
            field_dict[
                "useDefaultAccruePaymentInLieu"
            ] = use_default_accrue_payment_in_lieu
        if use_default_accrue_payment_in_lieu_rate is not UNSET:
            field_dict[
                "useDefaultAccruePaymentInLieuRate"
            ] = use_default_accrue_payment_in_lieu_rate
        if use_default_accrue_payment_in_lieu_all_gross_pay is not UNSET:
            field_dict[
                "useDefaultAccruePaymentInLieuAllGrossPay"
            ] = use_default_accrue_payment_in_lieu_all_gross_pay
        if use_default_accrue_payment_in_lieu_pay_automatically is not UNSET:
            field_dict[
                "useDefaultAccruePaymentInLieuPayAutomatically"
            ] = use_default_accrue_payment_in_lieu_pay_automatically
        if use_default_accrue_hours_per_day is not UNSET:
            field_dict["useDefaultAccrueHoursPerDay"] = use_default_accrue_hours_per_day
        if allowance_reset_date is not UNSET:
            field_dict["allowanceResetDate"] = allowance_reset_date
        if allowance is not UNSET:
            field_dict["allowance"] = allowance
        if adjustment is not UNSET:
            field_dict["adjustment"] = adjustment
        if allowance_used is not UNSET:
            field_dict["allowanceUsed"] = allowance_used
        if allowance_used_previous_period is not UNSET:
            field_dict["allowanceUsedPreviousPeriod"] = allowance_used_previous_period
        if allowance_remaining is not UNSET:
            field_dict["allowanceRemaining"] = allowance_remaining
        if holiday_type is not UNSET:
            field_dict["holidayType"] = holiday_type
        if accrue_set_amount is not UNSET:
            field_dict["accrueSetAmount"] = accrue_set_amount
        if accrue_hours_per_day is not UNSET:
            field_dict["accrueHoursPerDay"] = accrue_hours_per_day
        if show_allowance_on_payslip is not UNSET:
            field_dict["showAllowanceOnPayslip"] = show_allowance_on_payslip
        if show_ahp_on_payslip is not UNSET:
            field_dict["showAhpOnPayslip"] = show_ahp_on_payslip
        if accrue_payment_in_lieu_rate is not UNSET:
            field_dict["accruePaymentInLieuRate"] = accrue_payment_in_lieu_rate
        if accrue_payment_in_lieu_all_gross_pay is not UNSET:
            field_dict[
                "accruePaymentInLieuAllGrossPay"
            ] = accrue_payment_in_lieu_all_gross_pay
        if accrue_payment_in_lieu_pay_automatically is not UNSET:
            field_dict[
                "accruePaymentInLieuPayAutomatically"
            ] = accrue_payment_in_lieu_pay_automatically
        if accrued_payment_liability is not UNSET:
            field_dict["accruedPaymentLiability"] = accrued_payment_liability
        if accrued_payment_adjustment is not UNSET:
            field_dict["accruedPaymentAdjustment"] = accrued_payment_adjustment
        if accrued_payment_paid is not UNSET:
            field_dict["accruedPaymentPaid"] = accrued_payment_paid
        if accrued_payment_balance is not UNSET:
            field_dict["accruedPaymentBalance"] = accrued_payment_balance

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        use_default_holiday_type = d.pop("useDefaultHolidayType", UNSET)

        use_default_allowance_reset_date = d.pop("useDefaultAllowanceResetDate", UNSET)

        use_default_allowance = d.pop("useDefaultAllowance", UNSET)

        use_default_accrue_payment_in_lieu = d.pop(
            "useDefaultAccruePaymentInLieu", UNSET
        )

        use_default_accrue_payment_in_lieu_rate = d.pop(
            "useDefaultAccruePaymentInLieuRate", UNSET
        )

        use_default_accrue_payment_in_lieu_all_gross_pay = d.pop(
            "useDefaultAccruePaymentInLieuAllGrossPay", UNSET
        )

        use_default_accrue_payment_in_lieu_pay_automatically = d.pop(
            "useDefaultAccruePaymentInLieuPayAutomatically", UNSET
        )

        use_default_accrue_hours_per_day = d.pop("useDefaultAccrueHoursPerDay", UNSET)

        _allowance_reset_date = d.pop("allowanceResetDate", UNSET)
        allowance_reset_date: Union[Unset, datetime.date]
        if isinstance(_allowance_reset_date, Unset):
            allowance_reset_date = UNSET
        else:
            allowance_reset_date = isoparse(_allowance_reset_date).date()

        allowance = d.pop("allowance", UNSET)

        adjustment = d.pop("adjustment", UNSET)

        allowance_used = d.pop("allowanceUsed", UNSET)

        allowance_used_previous_period = d.pop("allowanceUsedPreviousPeriod", UNSET)

        allowance_remaining = d.pop("allowanceRemaining", UNSET)

        _holiday_type = d.pop("holidayType", UNSET)
        holiday_type: Union[Unset, HolidayType]
        if isinstance(_holiday_type, Unset):
            holiday_type = UNSET
        else:
            holiday_type = HolidayType(_holiday_type)

        accrue_set_amount = d.pop("accrueSetAmount", UNSET)

        accrue_hours_per_day = d.pop("accrueHoursPerDay", UNSET)

        show_allowance_on_payslip = d.pop("showAllowanceOnPayslip", UNSET)

        show_ahp_on_payslip = d.pop("showAhpOnPayslip", UNSET)

        accrue_payment_in_lieu_rate = d.pop("accruePaymentInLieuRate", UNSET)

        accrue_payment_in_lieu_all_gross_pay = d.pop(
            "accruePaymentInLieuAllGrossPay", UNSET
        )

        accrue_payment_in_lieu_pay_automatically = d.pop(
            "accruePaymentInLieuPayAutomatically", UNSET
        )

        accrued_payment_liability = d.pop("accruedPaymentLiability", UNSET)

        accrued_payment_adjustment = d.pop("accruedPaymentAdjustment", UNSET)

        accrued_payment_paid = d.pop("accruedPaymentPaid", UNSET)

        accrued_payment_balance = d.pop("accruedPaymentBalance", UNSET)

        leave_settings = cls(
            use_default_holiday_type=use_default_holiday_type,
            use_default_allowance_reset_date=use_default_allowance_reset_date,
            use_default_allowance=use_default_allowance,
            use_default_accrue_payment_in_lieu=use_default_accrue_payment_in_lieu,
            use_default_accrue_payment_in_lieu_rate=use_default_accrue_payment_in_lieu_rate,
            use_default_accrue_payment_in_lieu_all_gross_pay=use_default_accrue_payment_in_lieu_all_gross_pay,
            use_default_accrue_payment_in_lieu_pay_automatically=use_default_accrue_payment_in_lieu_pay_automatically,
            use_default_accrue_hours_per_day=use_default_accrue_hours_per_day,
            allowance_reset_date=allowance_reset_date,
            allowance=allowance,
            adjustment=adjustment,
            allowance_used=allowance_used,
            allowance_used_previous_period=allowance_used_previous_period,
            allowance_remaining=allowance_remaining,
            holiday_type=holiday_type,
            accrue_set_amount=accrue_set_amount,
            accrue_hours_per_day=accrue_hours_per_day,
            show_allowance_on_payslip=show_allowance_on_payslip,
            show_ahp_on_payslip=show_ahp_on_payslip,
            accrue_payment_in_lieu_rate=accrue_payment_in_lieu_rate,
            accrue_payment_in_lieu_all_gross_pay=accrue_payment_in_lieu_all_gross_pay,
            accrue_payment_in_lieu_pay_automatically=accrue_payment_in_lieu_pay_automatically,
            accrued_payment_liability=accrued_payment_liability,
            accrued_payment_adjustment=accrued_payment_adjustment,
            accrued_payment_paid=accrued_payment_paid,
            accrued_payment_balance=accrued_payment_balance,
        )

        return leave_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.html_email_settings import HtmlEmailSettings
from ..models.smtp_settings import SmtpSettings
from ..types import UNSET, Unset

T = TypeVar("T", bound="MailSettings")


@attr.s(auto_attribs=True)
class MailSettings:
    """Determines the settings used when the Employer sends emails.
    If CustomiseSmtpSettings is false then SmtpSettings will be null and our default internal settings will be used;

        Attributes:
            sender_name (str):
            sender_email (str):
            customise_smtp_settings (bool):
            customise_html_email_settings (bool):
            use_system_from_address (Union[Unset, bool]): If set to true then the SenderEmail provided will only be used in
                the
                ReplyTo fields. The system defaults will be used for the From address.
            sender_email_verified (Union[Unset, bool]): [readonly] The SenderEmail must be verified before you can send
                email using that address
            smtp_settings (Union[Unset, SmtpSettings]):
            html_email_settings (Union[Unset, HtmlEmailSettings]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    sender_name: str
    sender_email: str
    customise_smtp_settings: bool
    customise_html_email_settings: bool
    use_system_from_address: Union[Unset, bool] = UNSET
    sender_email_verified: Union[Unset, bool] = UNSET
    smtp_settings: Union[Unset, SmtpSettings] = UNSET
    html_email_settings: Union[Unset, HtmlEmailSettings] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        sender_name = self.sender_name
        sender_email = self.sender_email
        customise_smtp_settings = self.customise_smtp_settings
        customise_html_email_settings = self.customise_html_email_settings
        use_system_from_address = self.use_system_from_address
        sender_email_verified = self.sender_email_verified
        smtp_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.smtp_settings, Unset):
            smtp_settings = self.smtp_settings.to_dict()

        html_email_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.html_email_settings, Unset):
            html_email_settings = self.html_email_settings.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "senderName": sender_name,
                "senderEmail": sender_email,
                "customiseSmtpSettings": customise_smtp_settings,
                "customiseHtmlEmailSettings": customise_html_email_settings,
            }
        )
        if use_system_from_address is not UNSET:
            field_dict["useSystemFromAddress"] = use_system_from_address
        if sender_email_verified is not UNSET:
            field_dict["senderEmailVerified"] = sender_email_verified
        if smtp_settings is not UNSET:
            field_dict["smtpSettings"] = smtp_settings
        if html_email_settings is not UNSET:
            field_dict["htmlEmailSettings"] = html_email_settings
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sender_name = d.pop("senderName")

        sender_email = d.pop("senderEmail")

        customise_smtp_settings = d.pop("customiseSmtpSettings")

        customise_html_email_settings = d.pop("customiseHtmlEmailSettings")

        use_system_from_address = d.pop("useSystemFromAddress", UNSET)

        sender_email_verified = d.pop("senderEmailVerified", UNSET)

        _smtp_settings = d.pop("smtpSettings", UNSET)
        smtp_settings: Union[Unset, SmtpSettings]
        if isinstance(_smtp_settings, Unset):
            smtp_settings = UNSET
        else:
            smtp_settings = SmtpSettings.from_dict(_smtp_settings)

        _html_email_settings = d.pop("htmlEmailSettings", UNSET)
        html_email_settings: Union[Unset, HtmlEmailSettings]
        if isinstance(_html_email_settings, Unset):
            html_email_settings = UNSET
        else:
            html_email_settings = HtmlEmailSettings.from_dict(_html_email_settings)

        id = d.pop("id", UNSET)

        mail_settings = cls(
            sender_name=sender_name,
            sender_email=sender_email,
            customise_smtp_settings=customise_smtp_settings,
            customise_html_email_settings=customise_html_email_settings,
            use_system_from_address=use_system_from_address,
            sender_email_verified=sender_email_verified,
            smtp_settings=smtp_settings,
            html_email_settings=html_email_settings,
            id=id,
        )

        return mail_settings



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.p11d_desc_other import P11DDescOther
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DPaymentCollection")


@attr.s(auto_attribs=True)
class P11DPaymentCollection:
    """
    Attributes:
        payment (Union[Unset, None, List[P11DDescOther]]):
        tax (Union[Unset, None, str]):
        type_letter (Union[Unset, None, str]):
    """

    payment: Union[Unset, None, List[P11DDescOther]] = UNSET
    tax: Union[Unset, None, str] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        payment: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.payment, Unset):
            if self.payment is None:
                payment = None
            else:
                payment = []
                for payment_item_data in self.payment:
                    payment_item = payment_item_data.to_dict()

                    payment.append(payment_item)

        tax = self.tax
        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if payment is not UNSET:
            field_dict["payment"] = payment
        if tax is not UNSET:
            field_dict["tax"] = tax
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payment = []
        _payment = d.pop("payment", UNSET)
        for payment_item_data in _payment or []:
            payment_item = P11DDescOther.from_dict(payment_item_data)

            payment.append(payment_item)

        tax = d.pop("tax", UNSET)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_payment_collection = cls(
            payment=payment,
            tax=tax,
            type_letter=type_letter,
        )

        return p11d_payment_collection



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.exb_p11_db_class_1a_adjustments import ExbP11DbClass1AAdjustments
from ..models.exb_p11_db_class_1a_total_benefit import ExbP11DbClass1ATotalBenefit
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbP11DbClass1A")


@attr.s(auto_attribs=True)
class ExbP11DbClass1A:
    """
    Attributes:
        ni_cs_rate (Union[Unset, None, str]):
        total_benefit (Union[Unset, ExbP11DbClass1ATotalBenefit]):
        adjustments (Union[Unset, ExbP11DbClass1AAdjustments]):
        ni_cpayable (Union[Unset, None, str]):
    """

    ni_cs_rate: Union[Unset, None, str] = UNSET
    total_benefit: Union[Unset, ExbP11DbClass1ATotalBenefit] = UNSET
    adjustments: Union[Unset, ExbP11DbClass1AAdjustments] = UNSET
    ni_cpayable: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ni_cs_rate = self.ni_cs_rate
        total_benefit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.total_benefit, Unset):
            total_benefit = self.total_benefit.to_dict()

        adjustments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.adjustments, Unset):
            adjustments = self.adjustments.to_dict()

        ni_cpayable = self.ni_cpayable

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ni_cs_rate is not UNSET:
            field_dict["niCsRate"] = ni_cs_rate
        if total_benefit is not UNSET:
            field_dict["totalBenefit"] = total_benefit
        if adjustments is not UNSET:
            field_dict["adjustments"] = adjustments
        if ni_cpayable is not UNSET:
            field_dict["niCpayable"] = ni_cpayable

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ni_cs_rate = d.pop("niCsRate", UNSET)

        _total_benefit = d.pop("totalBenefit", UNSET)
        total_benefit: Union[Unset, ExbP11DbClass1ATotalBenefit]
        if isinstance(_total_benefit, Unset):
            total_benefit = UNSET
        else:
            total_benefit = ExbP11DbClass1ATotalBenefit.from_dict(_total_benefit)

        _adjustments = d.pop("adjustments", UNSET)
        adjustments: Union[Unset, ExbP11DbClass1AAdjustments]
        if isinstance(_adjustments, Unset):
            adjustments = UNSET
        else:
            adjustments = ExbP11DbClass1AAdjustments.from_dict(_adjustments)

        ni_cpayable = d.pop("niCpayable", UNSET)

        exb_p11_db_class_1a = cls(
            ni_cs_rate=ni_cs_rate,
            total_benefit=total_benefit,
            adjustments=adjustments,
            ni_cpayable=ni_cpayable,
        )

        return exb_p11_db_class_1a



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.papdis_message_function_code import PapdisMessageFunctionCode
from ..models.papdis_pension_provider import PapdisPensionProvider
from ..models.papdis_version import PapdisVersion
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisDocument")


@attr.s(auto_attribs=True)
class PapdisDocument:
    """PAPDIS stands for 'Payroll and Pension Data Interface Standard'.
    It is an industry standard for exchanging data between payroll software and pension providers.
    Our system provides an export of data in this standard and these models are used to represent the data.
    Whilst the relevant reporting API endpoint can provide this data as a JSON entity, it is usually represented in CSV
    or XML format which our API also provides.

        Attributes:
            message_function_code (Union[Unset, PapdisMessageFunctionCode]):
            version (Union[Unset, PapdisVersion]):
            message_function_code_int (Union[Unset, int]): [readonly]
            pension_provider (Union[Unset, PapdisPensionProvider]):
            report (Union[Unset, Report]):
            tax_year (Union[Unset, TaxYear]):
    """

    message_function_code: Union[Unset, PapdisMessageFunctionCode] = UNSET
    version: Union[Unset, PapdisVersion] = UNSET
    message_function_code_int: Union[Unset, int] = UNSET
    pension_provider: Union[Unset, PapdisPensionProvider] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        message_function_code: Union[Unset, str] = UNSET
        if not isinstance(self.message_function_code, Unset):
            message_function_code = self.message_function_code.value

        version: Union[Unset, str] = UNSET
        if not isinstance(self.version, Unset):
            version = self.version.value

        message_function_code_int = self.message_function_code_int
        pension_provider: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pension_provider, Unset):
            pension_provider = self.pension_provider.to_dict()

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if message_function_code is not UNSET:
            field_dict["messageFunctionCode"] = message_function_code
        if version is not UNSET:
            field_dict["version"] = version
        if message_function_code_int is not UNSET:
            field_dict["messageFunctionCodeInt"] = message_function_code_int
        if pension_provider is not UNSET:
            field_dict["pensionProvider"] = pension_provider
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _message_function_code = d.pop("messageFunctionCode", UNSET)
        message_function_code: Union[Unset, PapdisMessageFunctionCode]
        if isinstance(_message_function_code, Unset):
            message_function_code = UNSET
        else:
            message_function_code = PapdisMessageFunctionCode(_message_function_code)

        _version = d.pop("version", UNSET)
        version: Union[Unset, PapdisVersion]
        if isinstance(_version, Unset):
            version = UNSET
        else:
            version = PapdisVersion(_version)

        message_function_code_int = d.pop("messageFunctionCodeInt", UNSET)

        _pension_provider = d.pop("pensionProvider", UNSET)
        pension_provider: Union[Unset, PapdisPensionProvider]
        if isinstance(_pension_provider, Unset):
            pension_provider = UNSET
        else:
            pension_provider = PapdisPensionProvider.from_dict(_pension_provider)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        papdis_document = cls(
            message_function_code=message_function_code,
            version=version,
            message_function_code_int=message_function_code_int,
            pension_provider=pension_provider,
            report=report,
            tax_year=tax_year,
        )

        return papdis_document



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="HourlyPayReportLine")


@attr.s(auto_attribs=True)
class HourlyPayReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        pay_code (Union[Unset, None, str]):
        period (Union[Unset, int]):
        hours (Union[Unset, float]):
        rate (Union[Unset, float]):
        total (Union[Unset, float]):
    """

    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    pay_code: Union[Unset, None, str] = UNSET
    period: Union[Unset, int] = UNSET
    hours: Union[Unset, float] = UNSET
    rate: Union[Unset, float] = UNSET
    total: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        pay_code = self.pay_code
        period = self.period
        hours = self.hours
        rate = self.rate
        total = self.total

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if period is not UNSET:
            field_dict["period"] = period
        if hours is not UNSET:
            field_dict["hours"] = hours
        if rate is not UNSET:
            field_dict["rate"] = rate
        if total is not UNSET:
            field_dict["total"] = total

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        pay_code = d.pop("payCode", UNSET)

        period = d.pop("period", UNSET)

        hours = d.pop("hours", UNSET)

        rate = d.pop("rate", UNSET)

        total = d.pop("total", UNSET)

        hourly_pay_report_line = cls(
            employee=employee,
            payroll_code=payroll_code,
            pay_code=pay_code,
            period=period,
            hours=hours,
            rate=rate,
            total=total,
        )

        return hourly_pay_report_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExternalDataCompany")


@attr.s(auto_attribs=True)
class ExternalDataCompany:
    """When we retrieve data from an ExternalDataProvider we normalise it so that regardless of the provider the models are
    the same.
    This model is used to represent a Company in an ExternalDataProvider

        Attributes:
            id (Union[Unset, None, str]):
            name (Union[Unset, None, str]):
            scheme_name (Union[Unset, None, str]):
    """

    id: Union[Unset, None, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    scheme_name: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        scheme_name = self.scheme_name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if scheme_name is not UNSET:
            field_dict["schemeName"] = scheme_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        scheme_name = d.pop("schemeName", UNSET)

        external_data_company = cls(
            id=id,
            name=name,
            scheme_name=scheme_name,
        )

        return external_data_company



from typing import Any, Dict

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    client: Client,
    json_body: str,
) -> Dict[str, Any]:
    url = "{}/employers/{id}/users/owner".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    id: str,
    *,
    client: Client,
    json_body: str,
) -> Response[Any]:
    """Set Owner

     Sets the owner for the employer. You must be the owner of the Employer to set this value.
    The user you are setting as the owner must already have the role Admin.
    You can get a list of users from the ListUsers endpoint.

    Args:
        id (str):
        json_body (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    id: str,
    *,
    client: Client,
    json_body: str,
) -> Response[Any]:
    """Set Owner

     Sets the owner for the employer. You must be the owner of the Employer to set this value.
    The user you are setting as the owner must already have the role Admin.
    You can get a list of users from the ListUsers endpoint.

    Args:
        id (str):
        json_body (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeTaxCode")


@attr.s(auto_attribs=True)
class FpsEmployeeTaxCode:
    """
    Attributes:
        basis_non_cumulative (Union[Unset, None, str]):
        tax_regime (Union[Unset, None, str]):
        tax_code (Union[Unset, None, str]):
    """

    basis_non_cumulative: Union[Unset, None, str] = UNSET
    tax_regime: Union[Unset, None, str] = UNSET
    tax_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        basis_non_cumulative = self.basis_non_cumulative
        tax_regime = self.tax_regime
        tax_code = self.tax_code

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if basis_non_cumulative is not UNSET:
            field_dict["basisNonCumulative"] = basis_non_cumulative
        if tax_regime is not UNSET:
            field_dict["taxRegime"] = tax_regime
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        basis_non_cumulative = d.pop("basisNonCumulative", UNSET)

        tax_regime = d.pop("taxRegime", UNSET)

        tax_code = d.pop("taxCode", UNSET)

        fps_employee_tax_code = cls(
            basis_non_cumulative=basis_non_cumulative,
            tax_regime=tax_regime,
            tax_code=tax_code,
        )

        return fps_employee_tax_code



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PaySchedulePeriodEventsConfig")


@attr.s(auto_attribs=True)
class PaySchedulePeriodEventsConfig:
    """Only applicable if Bureau functionality is enabled. Defines the number of days each event occurs before the Payment
    Date.

        Attributes:
            submit_for_processing (Union[Unset, int]):
            send_for_approval (Union[Unset, int]):
            approval (Union[Unset, int]):
            finalise (Union[Unset, int]):
            send_pay_slip (Union[Unset, int]):
            submit_rti (Union[Unset, int]):
    """

    submit_for_processing: Union[Unset, int] = UNSET
    send_for_approval: Union[Unset, int] = UNSET
    approval: Union[Unset, int] = UNSET
    finalise: Union[Unset, int] = UNSET
    send_pay_slip: Union[Unset, int] = UNSET
    submit_rti: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        submit_for_processing = self.submit_for_processing
        send_for_approval = self.send_for_approval
        approval = self.approval
        finalise = self.finalise
        send_pay_slip = self.send_pay_slip
        submit_rti = self.submit_rti

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if submit_for_processing is not UNSET:
            field_dict["SubmitForProcessing"] = submit_for_processing
        if send_for_approval is not UNSET:
            field_dict["SendForApproval"] = send_for_approval
        if approval is not UNSET:
            field_dict["Approval"] = approval
        if finalise is not UNSET:
            field_dict["Finalise"] = finalise
        if send_pay_slip is not UNSET:
            field_dict["SendPaySlip"] = send_pay_slip
        if submit_rti is not UNSET:
            field_dict["SubmitRti"] = submit_rti

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        submit_for_processing = d.pop("SubmitForProcessing", UNSET)

        send_for_approval = d.pop("SendForApproval", UNSET)

        approval = d.pop("Approval", UNSET)

        finalise = d.pop("Finalise", UNSET)

        send_pay_slip = d.pop("SendPaySlip", UNSET)

        submit_rti = d.pop("SubmitRti", UNSET)

        pay_schedule_period_events_config = cls(
            submit_for_processing=submit_for_processing,
            send_for_approval=send_for_approval,
            approval=approval,
            finalise=finalise,
            send_pay_slip=send_pay_slip,
            submit_rti=submit_rti,
        )

        return pay_schedule_period_events_config



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.analysis_report_line_value import AnalysisReportLineValue
from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalysisReportLine")


@attr.s(auto_attribs=True)
class AnalysisReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        gross (Union[Unset, float]):
        niable_gross (Union[Unset, float]):
        pensionable_gross (Union[Unset, float]):
        net (Union[Unset, float]):
        take_home (Union[Unset, float]):
        employer_nic (Union[Unset, float]):
        employer_pension (Union[Unset, float]):
        show_qty_column (Union[Unset, bool]):
        lines (Union[Unset, None, List[AnalysisReportLineValue]]):
    """

    employee: Union[Unset, Item] = UNSET
    gross: Union[Unset, float] = UNSET
    niable_gross: Union[Unset, float] = UNSET
    pensionable_gross: Union[Unset, float] = UNSET
    net: Union[Unset, float] = UNSET
    take_home: Union[Unset, float] = UNSET
    employer_nic: Union[Unset, float] = UNSET
    employer_pension: Union[Unset, float] = UNSET
    show_qty_column: Union[Unset, bool] = UNSET
    lines: Union[Unset, None, List[AnalysisReportLineValue]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        gross = self.gross
        niable_gross = self.niable_gross
        pensionable_gross = self.pensionable_gross
        net = self.net
        take_home = self.take_home
        employer_nic = self.employer_nic
        employer_pension = self.employer_pension
        show_qty_column = self.show_qty_column
        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if gross is not UNSET:
            field_dict["gross"] = gross
        if niable_gross is not UNSET:
            field_dict["niableGross"] = niable_gross
        if pensionable_gross is not UNSET:
            field_dict["pensionableGross"] = pensionable_gross
        if net is not UNSET:
            field_dict["net"] = net
        if take_home is not UNSET:
            field_dict["takeHome"] = take_home
        if employer_nic is not UNSET:
            field_dict["employerNic"] = employer_nic
        if employer_pension is not UNSET:
            field_dict["employerPension"] = employer_pension
        if show_qty_column is not UNSET:
            field_dict["showQtyColumn"] = show_qty_column
        if lines is not UNSET:
            field_dict["lines"] = lines

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        gross = d.pop("gross", UNSET)

        niable_gross = d.pop("niableGross", UNSET)

        pensionable_gross = d.pop("pensionableGross", UNSET)

        net = d.pop("net", UNSET)

        take_home = d.pop("takeHome", UNSET)

        employer_nic = d.pop("employerNic", UNSET)

        employer_pension = d.pop("employerPension", UNSET)

        show_qty_column = d.pop("showQtyColumn", UNSET)

        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = AnalysisReportLineValue.from_dict(lines_item_data)

            lines.append(lines_item)

        analysis_report_line = cls(
            employee=employee,
            gross=gross,
            niable_gross=niable_gross,
            pensionable_gross=pensionable_gross,
            net=net,
            take_home=take_home,
            employer_nic=employer_nic,
            employer_pension=employer_pension,
            show_qty_column=show_qty_column,
            lines=lines,
        )

        return analysis_report_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pension_scheme import PensionScheme
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionSelection")


@attr.s(auto_attribs=True)
class PensionSelection:
    """
    Attributes:
        id (Union[Unset, str]): [readonly] The unique id of the object
        pension_scheme_id (Union[Unset, str]):
        pension_scheme (Union[Unset, PensionScheme]):
        worker_group_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    pension_scheme_id: Union[Unset, str] = UNSET
    pension_scheme: Union[Unset, PensionScheme] = UNSET
    worker_group_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        pension_scheme_id = self.pension_scheme_id
        pension_scheme: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pension_scheme, Unset):
            pension_scheme = self.pension_scheme.to_dict()

        worker_group_id = self.worker_group_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if pension_scheme_id is not UNSET:
            field_dict["pensionSchemeId"] = pension_scheme_id
        if pension_scheme is not UNSET:
            field_dict["pensionScheme"] = pension_scheme
        if worker_group_id is not UNSET:
            field_dict["workerGroupId"] = worker_group_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        pension_scheme_id = d.pop("pensionSchemeId", UNSET)

        _pension_scheme = d.pop("pensionScheme", UNSET)
        pension_scheme: Union[Unset, PensionScheme]
        if isinstance(_pension_scheme, Unset):
            pension_scheme = UNSET
        else:
            pension_scheme = PensionScheme.from_dict(_pension_scheme)

        worker_group_id = d.pop("workerGroupId", UNSET)

        pension_selection = cls(
            id=id,
            pension_scheme_id=pension_scheme_id,
            pension_scheme=pension_scheme,
            worker_group_id=worker_group_id,
        )

        return pension_selection



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..models.leave_settings import LeaveSettings
from ..types import UNSET, Unset

T = TypeVar("T", bound="HolidayReportLine")


@attr.s(auto_attribs=True)
class HolidayReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
        day_rate (Union[Unset, None, float]):
        leave_settings (Union[Unset, LeaveSettings]):
    """

    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET
    day_rate: Union[Unset, None, float] = UNSET
    leave_settings: Union[Unset, LeaveSettings] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        department = self.department
        day_rate = self.day_rate
        leave_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.leave_settings, Unset):
            leave_settings = self.leave_settings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department
        if day_rate is not UNSET:
            field_dict["dayRate"] = day_rate
        if leave_settings is not UNSET:
            field_dict["leaveSettings"] = leave_settings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        day_rate = d.pop("dayRate", UNSET)

        _leave_settings = d.pop("leaveSettings", UNSET)
        leave_settings: Union[Unset, LeaveSettings]
        if isinstance(_leave_settings, Unset):
            leave_settings = UNSET
        else:
            leave_settings = LeaveSettings.from_dict(_leave_settings)

        holiday_report_line = cls(
            employee=employee,
            payroll_code=payroll_code,
            department=department,
            day_rate=day_rate,
            leave_settings=leave_settings,
        )

        return holiday_report_line



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.pay_basis import PayBasis
from ..models.payments_csv_mapping_column import PaymentsCsvMappingColumn
from ..models.payments_csv_mapping_type import PaymentsCsvMappingType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PaymentsCsvMapping")


@attr.s(auto_attribs=True)
class PaymentsCsvMapping:
    """This model is used to save CSV mappings for importing of payments.
    It probably has very little practical use outside of our own UI

        Attributes:
            name (str):
            type (Union[Unset, PaymentsCsvMappingType]):
            import_behaviour (Union[Unset, int]):
            pay_basis (Union[Unset, PayBasis]):
            has_header (Union[Unset, bool]):
            payroll_code_index (Union[Unset, int]):
            employer_id_index (Union[Unset, int]):
            pay_amount_index (Union[Unset, int]):
            pay_amount_multiplier_index (Union[Unset, int]):
            note_index (Union[Unset, int]):
            pay_code_index (Union[Unset, int]):
            columns (Union[Unset, None, List[PaymentsCsvMappingColumn]]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    name: str
    type: Union[Unset, PaymentsCsvMappingType] = UNSET
    import_behaviour: Union[Unset, int] = UNSET
    pay_basis: Union[Unset, PayBasis] = UNSET
    has_header: Union[Unset, bool] = UNSET
    payroll_code_index: Union[Unset, int] = UNSET
    employer_id_index: Union[Unset, int] = UNSET
    pay_amount_index: Union[Unset, int] = UNSET
    pay_amount_multiplier_index: Union[Unset, int] = UNSET
    note_index: Union[Unset, int] = UNSET
    pay_code_index: Union[Unset, int] = UNSET
    columns: Union[Unset, None, List[PaymentsCsvMappingColumn]] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        import_behaviour = self.import_behaviour
        pay_basis: Union[Unset, str] = UNSET
        if not isinstance(self.pay_basis, Unset):
            pay_basis = self.pay_basis.value

        has_header = self.has_header
        payroll_code_index = self.payroll_code_index
        employer_id_index = self.employer_id_index
        pay_amount_index = self.pay_amount_index
        pay_amount_multiplier_index = self.pay_amount_multiplier_index
        note_index = self.note_index
        pay_code_index = self.pay_code_index
        columns: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.columns, Unset):
            if self.columns is None:
                columns = None
            else:
                columns = []
                for columns_item_data in self.columns:
                    columns_item = columns_item_data.to_dict()

                    columns.append(columns_item)

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type
        if import_behaviour is not UNSET:
            field_dict["importBehaviour"] = import_behaviour
        if pay_basis is not UNSET:
            field_dict["payBasis"] = pay_basis
        if has_header is not UNSET:
            field_dict["hasHeader"] = has_header
        if payroll_code_index is not UNSET:
            field_dict["payrollCodeIndex"] = payroll_code_index
        if employer_id_index is not UNSET:
            field_dict["employerIdIndex"] = employer_id_index
        if pay_amount_index is not UNSET:
            field_dict["payAmountIndex"] = pay_amount_index
        if pay_amount_multiplier_index is not UNSET:
            field_dict["payAmountMultiplierIndex"] = pay_amount_multiplier_index
        if note_index is not UNSET:
            field_dict["noteIndex"] = note_index
        if pay_code_index is not UNSET:
            field_dict["payCodeIndex"] = pay_code_index
        if columns is not UNSET:
            field_dict["columns"] = columns
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        _type = d.pop("type", UNSET)
        type: Union[Unset, PaymentsCsvMappingType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PaymentsCsvMappingType(_type)

        import_behaviour = d.pop("importBehaviour", UNSET)

        _pay_basis = d.pop("payBasis", UNSET)
        pay_basis: Union[Unset, PayBasis]
        if isinstance(_pay_basis, Unset):
            pay_basis = UNSET
        else:
            pay_basis = PayBasis(_pay_basis)

        has_header = d.pop("hasHeader", UNSET)

        payroll_code_index = d.pop("payrollCodeIndex", UNSET)

        employer_id_index = d.pop("employerIdIndex", UNSET)

        pay_amount_index = d.pop("payAmountIndex", UNSET)

        pay_amount_multiplier_index = d.pop("payAmountMultiplierIndex", UNSET)

        note_index = d.pop("noteIndex", UNSET)

        pay_code_index = d.pop("payCodeIndex", UNSET)

        columns = []
        _columns = d.pop("columns", UNSET)
        for columns_item_data in _columns or []:
            columns_item = PaymentsCsvMappingColumn.from_dict(columns_item_data)

            columns.append(columns_item)

        id = d.pop("id", UNSET)

        payments_csv_mapping = cls(
            name=name,
            type=type,
            import_behaviour=import_behaviour,
            pay_basis=pay_basis,
            has_header=has_header,
            payroll_code_index=payroll_code_index,
            employer_id_index=employer_id_index,
            pay_amount_index=pay_amount_index,
            pay_amount_multiplier_index=pay_amount_multiplier_index,
            note_index=note_index,
            pay_code_index=pay_code_index,
            columns=columns,
            id=id,
        )

        return payments_csv_mapping



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cis_sub_contractor_type import CISSubContractorType
from ..models.cis_verification_details import CisVerificationDetails
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisDetails")


@attr.s(auto_attribs=True)
class CisDetails:
    """If an Employee is marked as a CIS Subcontractor then this model provides further details specifically related to the
    CIS details of the Employee.

        Attributes:
            type (Union[Unset, CISSubContractorType]):
            utr (Union[Unset, None, str]):
            trading_name (Union[Unset, None, str]):
            company_utr (Union[Unset, None, str]):
            company_number (Union[Unset, None, str]):
            vat_registered (Union[Unset, bool]):
            vat_number (Union[Unset, None, str]):
            vat_rate (Union[Unset, float]):
            reverse_charge_vat (Union[Unset, bool]):
            verification (Union[Unset, CisVerificationDetails]):
    """

    type: Union[Unset, CISSubContractorType] = UNSET
    utr: Union[Unset, None, str] = UNSET
    trading_name: Union[Unset, None, str] = UNSET
    company_utr: Union[Unset, None, str] = UNSET
    company_number: Union[Unset, None, str] = UNSET
    vat_registered: Union[Unset, bool] = UNSET
    vat_number: Union[Unset, None, str] = UNSET
    vat_rate: Union[Unset, float] = UNSET
    reverse_charge_vat: Union[Unset, bool] = UNSET
    verification: Union[Unset, CisVerificationDetails] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        utr = self.utr
        trading_name = self.trading_name
        company_utr = self.company_utr
        company_number = self.company_number
        vat_registered = self.vat_registered
        vat_number = self.vat_number
        vat_rate = self.vat_rate
        reverse_charge_vat = self.reverse_charge_vat
        verification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.verification, Unset):
            verification = self.verification.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if utr is not UNSET:
            field_dict["utr"] = utr
        if trading_name is not UNSET:
            field_dict["tradingName"] = trading_name
        if company_utr is not UNSET:
            field_dict["companyUtr"] = company_utr
        if company_number is not UNSET:
            field_dict["companyNumber"] = company_number
        if vat_registered is not UNSET:
            field_dict["vatRegistered"] = vat_registered
        if vat_number is not UNSET:
            field_dict["vatNumber"] = vat_number
        if vat_rate is not UNSET:
            field_dict["vatRate"] = vat_rate
        if reverse_charge_vat is not UNSET:
            field_dict["reverseChargeVAT"] = reverse_charge_vat
        if verification is not UNSET:
            field_dict["verification"] = verification

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, CISSubContractorType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CISSubContractorType(_type)

        utr = d.pop("utr", UNSET)

        trading_name = d.pop("tradingName", UNSET)

        company_utr = d.pop("companyUtr", UNSET)

        company_number = d.pop("companyNumber", UNSET)

        vat_registered = d.pop("vatRegistered", UNSET)

        vat_number = d.pop("vatNumber", UNSET)

        vat_rate = d.pop("vatRate", UNSET)

        reverse_charge_vat = d.pop("reverseChargeVAT", UNSET)

        _verification = d.pop("verification", UNSET)
        verification: Union[Unset, CisVerificationDetails]
        if isinstance(_verification, Unset):
            verification = UNSET
        else:
            verification = CisVerificationDetails.from_dict(_verification)

        cis_details = cls(
            type=type,
            utr=utr,
            trading_name=trading_name,
            company_utr=company_utr,
            company_number=company_number,
            vat_registered=vat_registered,
            vat_number=vat_number,
            vat_rate=vat_rate,
            reverse_charge_vat=reverse_charge_vat,
            verification=verification,
        )

        return cis_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeFigsToDate")


@attr.s(auto_attribs=True)
class FpsEmployeeFigsToDate:
    """
    Attributes:
        taxable_pay (Union[Unset, None, str]):
        total_tax (Union[Unset, None, str]):
        student_loans_td (Union[Unset, None, str]):
        postgrad_loans_td (Union[Unset, None, str]):
        benefits_taxed_via_payroll_ytd (Union[Unset, None, str]):
        empee_pen_contribns_paid_ytd (Union[Unset, None, str]):
        empee_pen_contribns_not_paid_ytd (Union[Unset, None, str]):
        smpytd (Union[Unset, None, str]):
        sppytd (Union[Unset, None, str]):
        sapytd (Union[Unset, None, str]):
        shppytd (Union[Unset, None, str]):
        spbpytd (Union[Unset, None, str]):
        sspytd (Union[Unset, None, str]):
        gross (Union[Unset, None, str]):
        net_pay (Union[Unset, None, str]):
        additions (Union[Unset, None, str]):
        deductions (Union[Unset, None, str]):
        take_home_pay (Union[Unset, None, str]):
        adjustments (Union[Unset, None, str]):
        maps_miles (Union[Unset, None, str]):
        pensionable_earnings (Union[Unset, None, str]):
        pensionable_pay (Union[Unset, None, str]):
        employer_pension_contribution (Union[Unset, None, str]):
        employee_pension_contribution (Union[Unset, None, str]):
        employee_pension_contribution_avc (Union[Unset, None, str]):
        payment_after_leaving (Union[Unset, None, str]):
        tax_on_payment_after_leaving (Union[Unset, None, str]):
        taxable_pay_previous_employment (Union[Unset, None, str]):
        total_tax_previous_employment (Union[Unset, None, str]):
    """

    taxable_pay: Union[Unset, None, str] = UNSET
    total_tax: Union[Unset, None, str] = UNSET
    student_loans_td: Union[Unset, None, str] = UNSET
    postgrad_loans_td: Union[Unset, None, str] = UNSET
    benefits_taxed_via_payroll_ytd: Union[Unset, None, str] = UNSET
    empee_pen_contribns_paid_ytd: Union[Unset, None, str] = UNSET
    empee_pen_contribns_not_paid_ytd: Union[Unset, None, str] = UNSET
    smpytd: Union[Unset, None, str] = UNSET
    sppytd: Union[Unset, None, str] = UNSET
    sapytd: Union[Unset, None, str] = UNSET
    shppytd: Union[Unset, None, str] = UNSET
    spbpytd: Union[Unset, None, str] = UNSET
    sspytd: Union[Unset, None, str] = UNSET
    gross: Union[Unset, None, str] = UNSET
    net_pay: Union[Unset, None, str] = UNSET
    additions: Union[Unset, None, str] = UNSET
    deductions: Union[Unset, None, str] = UNSET
    take_home_pay: Union[Unset, None, str] = UNSET
    adjustments: Union[Unset, None, str] = UNSET
    maps_miles: Union[Unset, None, str] = UNSET
    pensionable_earnings: Union[Unset, None, str] = UNSET
    pensionable_pay: Union[Unset, None, str] = UNSET
    employer_pension_contribution: Union[Unset, None, str] = UNSET
    employee_pension_contribution: Union[Unset, None, str] = UNSET
    employee_pension_contribution_avc: Union[Unset, None, str] = UNSET
    payment_after_leaving: Union[Unset, None, str] = UNSET
    tax_on_payment_after_leaving: Union[Unset, None, str] = UNSET
    taxable_pay_previous_employment: Union[Unset, None, str] = UNSET
    total_tax_previous_employment: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        taxable_pay = self.taxable_pay
        total_tax = self.total_tax
        student_loans_td = self.student_loans_td
        postgrad_loans_td = self.postgrad_loans_td
        benefits_taxed_via_payroll_ytd = self.benefits_taxed_via_payroll_ytd
        empee_pen_contribns_paid_ytd = self.empee_pen_contribns_paid_ytd
        empee_pen_contribns_not_paid_ytd = self.empee_pen_contribns_not_paid_ytd
        smpytd = self.smpytd
        sppytd = self.sppytd
        sapytd = self.sapytd
        shppytd = self.shppytd
        spbpytd = self.spbpytd
        sspytd = self.sspytd
        gross = self.gross
        net_pay = self.net_pay
        additions = self.additions
        deductions = self.deductions
        take_home_pay = self.take_home_pay
        adjustments = self.adjustments
        maps_miles = self.maps_miles
        pensionable_earnings = self.pensionable_earnings
        pensionable_pay = self.pensionable_pay
        employer_pension_contribution = self.employer_pension_contribution
        employee_pension_contribution = self.employee_pension_contribution
        employee_pension_contribution_avc = self.employee_pension_contribution_avc
        payment_after_leaving = self.payment_after_leaving
        tax_on_payment_after_leaving = self.tax_on_payment_after_leaving
        taxable_pay_previous_employment = self.taxable_pay_previous_employment
        total_tax_previous_employment = self.total_tax_previous_employment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if taxable_pay is not UNSET:
            field_dict["taxablePay"] = taxable_pay
        if total_tax is not UNSET:
            field_dict["totalTax"] = total_tax
        if student_loans_td is not UNSET:
            field_dict["studentLoansTD"] = student_loans_td
        if postgrad_loans_td is not UNSET:
            field_dict["postgradLoansTD"] = postgrad_loans_td
        if benefits_taxed_via_payroll_ytd is not UNSET:
            field_dict["benefitsTaxedViaPayrollYTD"] = benefits_taxed_via_payroll_ytd
        if empee_pen_contribns_paid_ytd is not UNSET:
            field_dict["empeePenContribnsPaidYTD"] = empee_pen_contribns_paid_ytd
        if empee_pen_contribns_not_paid_ytd is not UNSET:
            field_dict["empeePenContribnsNotPaidYTD"] = empee_pen_contribns_not_paid_ytd
        if smpytd is not UNSET:
            field_dict["smpytd"] = smpytd
        if sppytd is not UNSET:
            field_dict["sppytd"] = sppytd
        if sapytd is not UNSET:
            field_dict["sapytd"] = sapytd
        if shppytd is not UNSET:
            field_dict["shppytd"] = shppytd
        if spbpytd is not UNSET:
            field_dict["spbpytd"] = spbpytd
        if sspytd is not UNSET:
            field_dict["sspytd"] = sspytd
        if gross is not UNSET:
            field_dict["gross"] = gross
        if net_pay is not UNSET:
            field_dict["netPay"] = net_pay
        if additions is not UNSET:
            field_dict["additions"] = additions
        if deductions is not UNSET:
            field_dict["deductions"] = deductions
        if take_home_pay is not UNSET:
            field_dict["takeHomePay"] = take_home_pay
        if adjustments is not UNSET:
            field_dict["adjustments"] = adjustments
        if maps_miles is not UNSET:
            field_dict["mapsMiles"] = maps_miles
        if pensionable_earnings is not UNSET:
            field_dict["pensionableEarnings"] = pensionable_earnings
        if pensionable_pay is not UNSET:
            field_dict["pensionablePay"] = pensionable_pay
        if employer_pension_contribution is not UNSET:
            field_dict["employerPensionContribution"] = employer_pension_contribution
        if employee_pension_contribution is not UNSET:
            field_dict["employeePensionContribution"] = employee_pension_contribution
        if employee_pension_contribution_avc is not UNSET:
            field_dict[
                "employeePensionContributionAvc"
            ] = employee_pension_contribution_avc
        if payment_after_leaving is not UNSET:
            field_dict["paymentAfterLeaving"] = payment_after_leaving
        if tax_on_payment_after_leaving is not UNSET:
            field_dict["taxOnPaymentAfterLeaving"] = tax_on_payment_after_leaving
        if taxable_pay_previous_employment is not UNSET:
            field_dict["taxablePayPreviousEmployment"] = taxable_pay_previous_employment
        if total_tax_previous_employment is not UNSET:
            field_dict["totalTaxPreviousEmployment"] = total_tax_previous_employment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        taxable_pay = d.pop("taxablePay", UNSET)

        total_tax = d.pop("totalTax", UNSET)

        student_loans_td = d.pop("studentLoansTD", UNSET)

        postgrad_loans_td = d.pop("postgradLoansTD", UNSET)

        benefits_taxed_via_payroll_ytd = d.pop("benefitsTaxedViaPayrollYTD", UNSET)

        empee_pen_contribns_paid_ytd = d.pop("empeePenContribnsPaidYTD", UNSET)

        empee_pen_contribns_not_paid_ytd = d.pop("empeePenContribnsNotPaidYTD", UNSET)

        smpytd = d.pop("smpytd", UNSET)

        sppytd = d.pop("sppytd", UNSET)

        sapytd = d.pop("sapytd", UNSET)

        shppytd = d.pop("shppytd", UNSET)

        spbpytd = d.pop("spbpytd", UNSET)

        sspytd = d.pop("sspytd", UNSET)

        gross = d.pop("gross", UNSET)

        net_pay = d.pop("netPay", UNSET)

        additions = d.pop("additions", UNSET)

        deductions = d.pop("deductions", UNSET)

        take_home_pay = d.pop("takeHomePay", UNSET)

        adjustments = d.pop("adjustments", UNSET)

        maps_miles = d.pop("mapsMiles", UNSET)

        pensionable_earnings = d.pop("pensionableEarnings", UNSET)

        pensionable_pay = d.pop("pensionablePay", UNSET)

        employer_pension_contribution = d.pop("employerPensionContribution", UNSET)

        employee_pension_contribution = d.pop("employeePensionContribution", UNSET)

        employee_pension_contribution_avc = d.pop(
            "employeePensionContributionAvc", UNSET
        )

        payment_after_leaving = d.pop("paymentAfterLeaving", UNSET)

        tax_on_payment_after_leaving = d.pop("taxOnPaymentAfterLeaving", UNSET)

        taxable_pay_previous_employment = d.pop("taxablePayPreviousEmployment", UNSET)

        total_tax_previous_employment = d.pop("totalTaxPreviousEmployment", UNSET)

        fps_employee_figs_to_date = cls(
            taxable_pay=taxable_pay,
            total_tax=total_tax,
            student_loans_td=student_loans_td,
            postgrad_loans_td=postgrad_loans_td,
            benefits_taxed_via_payroll_ytd=benefits_taxed_via_payroll_ytd,
            empee_pen_contribns_paid_ytd=empee_pen_contribns_paid_ytd,
            empee_pen_contribns_not_paid_ytd=empee_pen_contribns_not_paid_ytd,
            smpytd=smpytd,
            sppytd=sppytd,
            sapytd=sapytd,
            shppytd=shppytd,
            spbpytd=spbpytd,
            sspytd=sspytd,
            gross=gross,
            net_pay=net_pay,
            additions=additions,
            deductions=deductions,
            take_home_pay=take_home_pay,
            adjustments=adjustments,
            maps_miles=maps_miles,
            pensionable_earnings=pensionable_earnings,
            pensionable_pay=pensionable_pay,
            employer_pension_contribution=employer_pension_contribution,
            employee_pension_contribution=employee_pension_contribution,
            employee_pension_contribution_avc=employee_pension_contribution_avc,
            payment_after_leaving=payment_after_leaving,
            tax_on_payment_after_leaving=tax_on_payment_after_leaving,
            taxable_pay_previous_employment=taxable_pay_previous_employment,
            total_tax_previous_employment=total_tax_previous_employment,
        )

        return fps_employee_figs_to_date



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.pension_contribution_level_type import PensionContributionLevelType
from ..models.tiered_pension_rate import TieredPensionRate
from ..types import UNSET, Unset

T = TypeVar("T", bound="TieredPension")


@attr.s(auto_attribs=True)
class TieredPension:
    """Part of the TaxYearConfig that our engine uses to calculate tiered pension contributions.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            type (Union[Unset, PensionContributionLevelType]):
            rates (Union[Unset, None, List[TieredPensionRate]]):
            employer_contrib_rate (Union[Unset, float]):
    """

    type: Union[Unset, PensionContributionLevelType] = UNSET
    rates: Union[Unset, None, List[TieredPensionRate]] = UNSET
    employer_contrib_rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        rates: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rates, Unset):
            if self.rates is None:
                rates = None
            else:
                rates = []
                for rates_item_data in self.rates:
                    rates_item = rates_item_data.to_dict()

                    rates.append(rates_item)

        employer_contrib_rate = self.employer_contrib_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if rates is not UNSET:
            field_dict["rates"] = rates
        if employer_contrib_rate is not UNSET:
            field_dict["employerContribRate"] = employer_contrib_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PensionContributionLevelType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PensionContributionLevelType(_type)

        rates = []
        _rates = d.pop("rates", UNSET)
        for rates_item_data in _rates or []:
            rates_item = TieredPensionRate.from_dict(rates_item_data)

            rates.append(rates_item)

        employer_contrib_rate = d.pop("employerContribRate", UNSET)

        tiered_pension = cls(
            type=type,
            rates=rates,
            employer_contrib_rate=employer_contrib_rate,
        )

        return tiered_pension



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerGroupMembership")


@attr.s(auto_attribs=True)
class EmployerGroupMembership:
    """Used to represent an Employers membership of a EmployerGroup

    Attributes:
        code (str): The Code for the EmployerGroup
        color (Union[Unset, None, str]): [readonly] The Color for the EmployerGroup
        title (Union[Unset, None, str]): [readonly] The Title for the EmployerGroup
        primary (Union[Unset, bool]): Set to true if this is the primary EmployerGroup for the Employer.
            Only one EmployerGroup can be set as the primary.
    """

    code: str
    color: Union[Unset, None, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    primary: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        color = self.color
        title = self.title
        primary = self.primary

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "code": code,
            }
        )
        if color is not UNSET:
            field_dict["color"] = color
        if title is not UNSET:
            field_dict["title"] = title
        if primary is not UNSET:
            field_dict["primary"] = primary

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        color = d.pop("color", UNSET)

        title = d.pop("title", UNSET)

        primary = d.pop("primary", UNSET)

        employer_group_membership = cls(
            code=code,
            color=color,
            title=title,
            primary=primary,
        )

        return employer_group_membership



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="YearEndTaxCodeChange")


@attr.s(auto_attribs=True)
class YearEndTaxCodeChange:
    """Forms part of the YearEnd model to list changes to Tax Codes

    Attributes:
        employee (Union[Unset, Item]):
        current_code (Union[Unset, None, str]): [readonly] The Employees current tax code
        new_code (Union[Unset, None, str]): [readonly] The new TaxCode for the employee
    """

    employee: Union[Unset, Item] = UNSET
    current_code: Union[Unset, None, str] = UNSET
    new_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        current_code = self.current_code
        new_code = self.new_code

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if current_code is not UNSET:
            field_dict["currentCode"] = current_code
        if new_code is not UNSET:
            field_dict["newCode"] = new_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        current_code = d.pop("currentCode", UNSET)

        new_code = d.pop("newCode", UNSET)

        year_end_tax_code_change = cls(
            employee=employee,
            current_code=current_code,
            new_code=new_code,
        )

        return year_end_tax_code_change



from enum import Enum


class RightToWorkDocumentType(str, Enum):
    OTHER = "Other"
    VISA = "Visa"
    PASSPORT = "Passport"
    BIRTHCERTIFICATE = "BirthCertificate"
    IDENTITYCARD = "IdentityCard"
    SHARECODE = "ShareCode"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.average_weekly_earnings import AverageWeeklyEarnings
from ...models.leave_type import LeaveType
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    id: str,
    *,
    client: Client,
    date: Union[Unset, None, datetime.datetime] = UNSET,
    leave_type: Union[Unset, None, LeaveType] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{id}/awe".format(
        client.base_url, employerId=employer_id, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_date: Union[Unset, None, str] = UNSET
    if not isinstance(date, Unset):
        json_date = date.isoformat() if date else None

    params["date"] = json_date

    json_leave_type: Union[Unset, None, str] = UNSET
    if not isinstance(leave_type, Unset):
        json_leave_type = leave_type.value if leave_type else None

    params["leaveType"] = json_leave_type

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[AverageWeeklyEarnings]:
    if response.status_code == 200:
        response_200 = AverageWeeklyEarnings.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[AverageWeeklyEarnings]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    id: str,
    *,
    client: Client,
    date: Union[Unset, None, datetime.datetime] = UNSET,
    leave_type: Union[Unset, None, LeaveType] = UNSET,
) -> Response[AverageWeeklyEarnings]:
    """Average Weekly Earnings

     Calculates the Average Weekly Earnings for an Employee at the given date.
    You can optionally include a LeaveType and we'll adjust the given date accordingly
    For example if you specify Maternity then it'll be moved back by 15 weeks

    Args:
        employer_id (str):
        id (str):
        date (Union[Unset, None, datetime.datetime]):
        leave_type (Union[Unset, None, LeaveType]):

    Returns:
        Response[AverageWeeklyEarnings]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        date=date,
        leave_type=leave_type,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    id: str,
    *,
    client: Client,
    date: Union[Unset, None, datetime.datetime] = UNSET,
    leave_type: Union[Unset, None, LeaveType] = UNSET,
) -> Optional[AverageWeeklyEarnings]:
    """Average Weekly Earnings

     Calculates the Average Weekly Earnings for an Employee at the given date.
    You can optionally include a LeaveType and we'll adjust the given date accordingly
    For example if you specify Maternity then it'll be moved back by 15 weeks

    Args:
        employer_id (str):
        id (str):
        date (Union[Unset, None, datetime.datetime]):
        leave_type (Union[Unset, None, LeaveType]):

    Returns:
        Response[AverageWeeklyEarnings]
    """

    return sync_detailed(
        employer_id=employer_id,
        id=id,
        client=client,
        date=date,
        leave_type=leave_type,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    id: str,
    *,
    client: Client,
    date: Union[Unset, None, datetime.datetime] = UNSET,
    leave_type: Union[Unset, None, LeaveType] = UNSET,
) -> Response[AverageWeeklyEarnings]:
    """Average Weekly Earnings

     Calculates the Average Weekly Earnings for an Employee at the given date.
    You can optionally include a LeaveType and we'll adjust the given date accordingly
    For example if you specify Maternity then it'll be moved back by 15 weeks

    Args:
        employer_id (str):
        id (str):
        date (Union[Unset, None, datetime.datetime]):
        leave_type (Union[Unset, None, LeaveType]):

    Returns:
        Response[AverageWeeklyEarnings]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        date=date,
        leave_type=leave_type,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    id: str,
    *,
    client: Client,
    date: Union[Unset, None, datetime.datetime] = UNSET,
    leave_type: Union[Unset, None, LeaveType] = UNSET,
) -> Optional[AverageWeeklyEarnings]:
    """Average Weekly Earnings

     Calculates the Average Weekly Earnings for an Employee at the given date.
    You can optionally include a LeaveType and we'll adjust the given date accordingly
    For example if you specify Maternity then it'll be moved back by 15 weeks

    Args:
        employer_id (str):
        id (str):
        date (Union[Unset, None, datetime.datetime]):
        leave_type (Union[Unset, None, LeaveType]):

    Returns:
        Response[AverageWeeklyEarnings]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            id=id,
            client=client,
            date=date,
            leave_type=leave_type,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.employer_template_type import EmployerTemplateType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayrunEmail")


@attr.s(auto_attribs=True)
class PayrunEmail:
    """PayrunEmails are sent automatically when a payrun has been finalised

    Attributes:
        name (str): Only used for reference, not included in the email
        recipient_address (str): The address to which the email should be sent
        is_active (Union[Unset, bool]): If false then this email won't be sent
        template_type (Union[Unset, EmployerTemplateType]):
        pdf_password (Union[Unset, None, str]): If a value is provided, then all PDFs attached to the email will be
            encrypted using this password
        attach_payslips (Union[Unset, bool]): If set to true then Payslips will be attached to the email
        payslips_unemailed (Union[Unset, bool]): If AttachPayslips is true and this property is also true then only
            payslips that haven't already been sent directly to employees will be attached.
        payslips_single_file (Union[Unset, bool]): If AttachPayslips is true and this property is also true then the
            payslips will be attached as a single file rather than as separate PDFs,
        attach_payment_summary (Union[Unset, bool]): If set to True then the PaymentSummary report will be attached to
            the Email
        attach_p32 (Union[Unset, bool]): If set to True then the P32 report will be attached to the email
        attach_p30 (Union[Unset, bool]): If set to True then the P30 report will be attached to the email
        attach_gross_to_net (Union[Unset, bool]): If set to True then the GrossToNet report will be attached to the
            email
        attach_pension_contributions (Union[Unset, bool]): If set to True then the Pensions Contributions report will be
            attached to the email
        attach_cost_analysis (Union[Unset, bool]): If set to True then the Cost Analysis report will be attached to the
            email
        attach_cost_of_employment (Union[Unset, bool]): If set to True then the Cost of Employment report will be
            attached to the email
        attach_full_summary_of_pay (Union[Unset, bool]): If set to True then the Full Summary Of Payment report will be
            attached to the email
        attach_bank_payments (Union[Unset, bool]): If set to True then the Bank Payments CSV file will be attached to
            the email
        custom_subject (Union[Unset, None, str]): If TemplateType is null then you can provide a subject line here to be
            used for the email
        custom_body (Union[Unset, None, str]): If TemplateType is null then you can provide the body text here to be
            used for the email
        custom_body_is_html (Union[Unset, bool]): If the CustomBody is in HTML format, set this to true.
        child_id (Union[Unset, str]): This is nothing but the UniqueId of the model.
    """

    name: str
    recipient_address: str
    is_active: Union[Unset, bool] = UNSET
    template_type: Union[Unset, EmployerTemplateType] = UNSET
    pdf_password: Union[Unset, None, str] = UNSET
    attach_payslips: Union[Unset, bool] = UNSET
    payslips_unemailed: Union[Unset, bool] = UNSET
    payslips_single_file: Union[Unset, bool] = UNSET
    attach_payment_summary: Union[Unset, bool] = UNSET
    attach_p32: Union[Unset, bool] = UNSET
    attach_p30: Union[Unset, bool] = UNSET
    attach_gross_to_net: Union[Unset, bool] = UNSET
    attach_pension_contributions: Union[Unset, bool] = UNSET
    attach_cost_analysis: Union[Unset, bool] = UNSET
    attach_cost_of_employment: Union[Unset, bool] = UNSET
    attach_full_summary_of_pay: Union[Unset, bool] = UNSET
    attach_bank_payments: Union[Unset, bool] = UNSET
    custom_subject: Union[Unset, None, str] = UNSET
    custom_body: Union[Unset, None, str] = UNSET
    custom_body_is_html: Union[Unset, bool] = UNSET
    child_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        recipient_address = self.recipient_address
        is_active = self.is_active
        template_type: Union[Unset, str] = UNSET
        if not isinstance(self.template_type, Unset):
            template_type = self.template_type.value

        pdf_password = self.pdf_password
        attach_payslips = self.attach_payslips
        payslips_unemailed = self.payslips_unemailed
        payslips_single_file = self.payslips_single_file
        attach_payment_summary = self.attach_payment_summary
        attach_p32 = self.attach_p32
        attach_p30 = self.attach_p30
        attach_gross_to_net = self.attach_gross_to_net
        attach_pension_contributions = self.attach_pension_contributions
        attach_cost_analysis = self.attach_cost_analysis
        attach_cost_of_employment = self.attach_cost_of_employment
        attach_full_summary_of_pay = self.attach_full_summary_of_pay
        attach_bank_payments = self.attach_bank_payments
        custom_subject = self.custom_subject
        custom_body = self.custom_body
        custom_body_is_html = self.custom_body_is_html
        child_id = self.child_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "recipientAddress": recipient_address,
            }
        )
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if template_type is not UNSET:
            field_dict["templateType"] = template_type
        if pdf_password is not UNSET:
            field_dict["pdfPassword"] = pdf_password
        if attach_payslips is not UNSET:
            field_dict["attachPayslips"] = attach_payslips
        if payslips_unemailed is not UNSET:
            field_dict["payslips_Unemailed"] = payslips_unemailed
        if payslips_single_file is not UNSET:
            field_dict["payslips_SingleFile"] = payslips_single_file
        if attach_payment_summary is not UNSET:
            field_dict["attachPaymentSummary"] = attach_payment_summary
        if attach_p32 is not UNSET:
            field_dict["attachP32"] = attach_p32
        if attach_p30 is not UNSET:
            field_dict["attachP30"] = attach_p30
        if attach_gross_to_net is not UNSET:
            field_dict["attachGrossToNet"] = attach_gross_to_net
        if attach_pension_contributions is not UNSET:
            field_dict["attachPensionContributions"] = attach_pension_contributions
        if attach_cost_analysis is not UNSET:
            field_dict["attachCostAnalysis"] = attach_cost_analysis
        if attach_cost_of_employment is not UNSET:
            field_dict["attachCostOfEmployment"] = attach_cost_of_employment
        if attach_full_summary_of_pay is not UNSET:
            field_dict["attachFullSummaryOfPay"] = attach_full_summary_of_pay
        if attach_bank_payments is not UNSET:
            field_dict["attachBankPayments"] = attach_bank_payments
        if custom_subject is not UNSET:
            field_dict["customSubject"] = custom_subject
        if custom_body is not UNSET:
            field_dict["customBody"] = custom_body
        if custom_body_is_html is not UNSET:
            field_dict["customBody_IsHtml"] = custom_body_is_html
        if child_id is not UNSET:
            field_dict["childId"] = child_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        recipient_address = d.pop("recipientAddress")

        is_active = d.pop("isActive", UNSET)

        _template_type = d.pop("templateType", UNSET)
        template_type: Union[Unset, EmployerTemplateType]
        if isinstance(_template_type, Unset):
            template_type = UNSET
        else:
            template_type = EmployerTemplateType(_template_type)

        pdf_password = d.pop("pdfPassword", UNSET)

        attach_payslips = d.pop("attachPayslips", UNSET)

        payslips_unemailed = d.pop("payslips_Unemailed", UNSET)

        payslips_single_file = d.pop("payslips_SingleFile", UNSET)

        attach_payment_summary = d.pop("attachPaymentSummary", UNSET)

        attach_p32 = d.pop("attachP32", UNSET)

        attach_p30 = d.pop("attachP30", UNSET)

        attach_gross_to_net = d.pop("attachGrossToNet", UNSET)

        attach_pension_contributions = d.pop("attachPensionContributions", UNSET)

        attach_cost_analysis = d.pop("attachCostAnalysis", UNSET)

        attach_cost_of_employment = d.pop("attachCostOfEmployment", UNSET)

        attach_full_summary_of_pay = d.pop("attachFullSummaryOfPay", UNSET)

        attach_bank_payments = d.pop("attachBankPayments", UNSET)

        custom_subject = d.pop("customSubject", UNSET)

        custom_body = d.pop("customBody", UNSET)

        custom_body_is_html = d.pop("customBody_IsHtml", UNSET)

        child_id = d.pop("childId", UNSET)

        payrun_email = cls(
            name=name,
            recipient_address=recipient_address,
            is_active=is_active,
            template_type=template_type,
            pdf_password=pdf_password,
            attach_payslips=attach_payslips,
            payslips_unemailed=payslips_unemailed,
            payslips_single_file=payslips_single_file,
            attach_payment_summary=attach_payment_summary,
            attach_p32=attach_p32,
            attach_p30=attach_p30,
            attach_gross_to_net=attach_gross_to_net,
            attach_pension_contributions=attach_pension_contributions,
            attach_cost_analysis=attach_cost_analysis,
            attach_cost_of_employment=attach_cost_of_employment,
            attach_full_summary_of_pay=attach_full_summary_of_pay,
            attach_bank_payments=attach_bank_payments,
            custom_subject=custom_subject,
            custom_body=custom_body,
            custom_body_is_html=custom_body_is_html,
            child_id=child_id,
        )

        return payrun_email



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.report_response import ReportResponse
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    employer_id: Union[Unset, None, str] = UNSET,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    include_bank_account_details: Union[Unset, None, bool] = UNSET,
    include_ytd: Union[Unset, None, bool] = UNSET,
    inc_pension_info: Union[Unset, None, bool] = UNSET,
    include_pay_info: Union[Unset, None, bool] = UNSET,
    include_notes: Union[Unset, None, bool] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employees".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(accept, Unset):
        headers["accept"] = accept

    params: Dict[str, Any] = {}
    params["employerId"] = employer_id

    json_tax_year: Union[Unset, None, str] = UNSET
    if not isinstance(tax_year, Unset):
        json_tax_year = tax_year.value if tax_year else None

    params["taxYear"] = json_tax_year

    params["includeBankAccountDetails"] = include_bank_account_details

    params["includeYtd"] = include_ytd

    params["incPensionInfo"] = inc_pension_info

    params["includePayInfo"] = include_pay_info

    params["includeNotes"] = include_notes

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ReportResponse]:
    if response.status_code == 200:
        response_200 = ReportResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ReportResponse]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    employer_id: Union[Unset, None, str] = UNSET,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    include_bank_account_details: Union[Unset, None, bool] = UNSET,
    include_ytd: Union[Unset, None, bool] = UNSET,
    inc_pension_info: Union[Unset, None, bool] = UNSET,
    include_pay_info: Union[Unset, None, bool] = UNSET,
    include_notes: Union[Unset, None, bool] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Employee Export Details

     Returns a CSV file containing details for all Employees.

    Args:
        employer_id (Union[Unset, None, str]):
        tax_year (Union[Unset, None, TaxYear]):
        include_bank_account_details (Union[Unset, None, bool]):
        include_ytd (Union[Unset, None, bool]):
        inc_pension_info (Union[Unset, None, bool]):
        include_pay_info (Union[Unset, None, bool]):
        include_notes (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        employer_id=employer_id,
        tax_year=tax_year,
        include_bank_account_details=include_bank_account_details,
        include_ytd=include_ytd,
        inc_pension_info=inc_pension_info,
        include_pay_info=include_pay_info,
        include_notes=include_notes,
        accept=accept,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    employer_id: Union[Unset, None, str] = UNSET,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    include_bank_account_details: Union[Unset, None, bool] = UNSET,
    include_ytd: Union[Unset, None, bool] = UNSET,
    inc_pension_info: Union[Unset, None, bool] = UNSET,
    include_pay_info: Union[Unset, None, bool] = UNSET,
    include_notes: Union[Unset, None, bool] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Employee Export Details

     Returns a CSV file containing details for all Employees.

    Args:
        employer_id (Union[Unset, None, str]):
        tax_year (Union[Unset, None, TaxYear]):
        include_bank_account_details (Union[Unset, None, bool]):
        include_ytd (Union[Unset, None, bool]):
        inc_pension_info (Union[Unset, None, bool]):
        include_pay_info (Union[Unset, None, bool]):
        include_notes (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return sync_detailed(
        client=client,
        employer_id=employer_id,
        tax_year=tax_year,
        include_bank_account_details=include_bank_account_details,
        include_ytd=include_ytd,
        inc_pension_info=inc_pension_info,
        include_pay_info=include_pay_info,
        include_notes=include_notes,
        accept=accept,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    employer_id: Union[Unset, None, str] = UNSET,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    include_bank_account_details: Union[Unset, None, bool] = UNSET,
    include_ytd: Union[Unset, None, bool] = UNSET,
    inc_pension_info: Union[Unset, None, bool] = UNSET,
    include_pay_info: Union[Unset, None, bool] = UNSET,
    include_notes: Union[Unset, None, bool] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Employee Export Details

     Returns a CSV file containing details for all Employees.

    Args:
        employer_id (Union[Unset, None, str]):
        tax_year (Union[Unset, None, TaxYear]):
        include_bank_account_details (Union[Unset, None, bool]):
        include_ytd (Union[Unset, None, bool]):
        inc_pension_info (Union[Unset, None, bool]):
        include_pay_info (Union[Unset, None, bool]):
        include_notes (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        employer_id=employer_id,
        tax_year=tax_year,
        include_bank_account_details=include_bank_account_details,
        include_ytd=include_ytd,
        inc_pension_info=inc_pension_info,
        include_pay_info=include_pay_info,
        include_notes=include_notes,
        accept=accept,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    employer_id: Union[Unset, None, str] = UNSET,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    include_bank_account_details: Union[Unset, None, bool] = UNSET,
    include_ytd: Union[Unset, None, bool] = UNSET,
    inc_pension_info: Union[Unset, None, bool] = UNSET,
    include_pay_info: Union[Unset, None, bool] = UNSET,
    include_notes: Union[Unset, None, bool] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Employee Export Details

     Returns a CSV file containing details for all Employees.

    Args:
        employer_id (Union[Unset, None, str]):
        tax_year (Union[Unset, None, TaxYear]):
        include_bank_account_details (Union[Unset, None, bool]):
        include_ytd (Union[Unset, None, bool]):
        inc_pension_info (Union[Unset, None, bool]):
        include_pay_info (Union[Unset, None, bool]):
        include_notes (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            employer_id=employer_id,
            tax_year=tax_year,
            include_bank_account_details=include_bank_account_details,
            include_ytd=include_ytd,
            inc_pension_info=inc_pension_info,
            include_pay_info=include_pay_info,
            include_notes=include_notes,
            accept=accept,
        )
    ).parsed



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...models.csv_file_format import CsvFileFormat
from ...models.import_csv_employee_multipart_data import ImportCsvEmployeeMultipartData
from ...models.item import Item
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    *,
    client: Client,
    multipart_data: ImportCsvEmployeeMultipartData,
    format_: Union[Unset, None, CsvFileFormat] = UNSET,
    preview_only: Union[Unset, None, bool] = False,
    allow_updates: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/import".format(
        client.base_url, employerId=employer_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_format_: Union[Unset, None, str] = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value if format_ else None

    params["format"] = json_format_

    params["previewOnly"] = preview_only

    params["allowUpdates"] = allow_updates

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, List[Item]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 409:
        response_409 = cast(Any, None)
        return response_409
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, List[Item]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    *,
    client: Client,
    multipart_data: ImportCsvEmployeeMultipartData,
    format_: Union[Unset, None, CsvFileFormat] = UNSET,
    preview_only: Union[Unset, None, bool] = False,
    allow_updates: Union[Unset, None, bool] = False,
) -> Response[Union[Any, List[Item]]]:
    """Import CSV

     Import employee data from a CSV file.

    Args:
        employer_id (str):
        format_ (Union[Unset, None, CsvFileFormat]):
        preview_only (Union[Unset, None, bool]):
        allow_updates (Union[Unset, None, bool]):
        multipart_data (ImportCsvEmployeeMultipartData):

    Returns:
        Response[Union[Any, List[Item]]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        multipart_data=multipart_data,
        format_=format_,
        preview_only=preview_only,
        allow_updates=allow_updates,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    *,
    client: Client,
    multipart_data: ImportCsvEmployeeMultipartData,
    format_: Union[Unset, None, CsvFileFormat] = UNSET,
    preview_only: Union[Unset, None, bool] = False,
    allow_updates: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, List[Item]]]:
    """Import CSV

     Import employee data from a CSV file.

    Args:
        employer_id (str):
        format_ (Union[Unset, None, CsvFileFormat]):
        preview_only (Union[Unset, None, bool]):
        allow_updates (Union[Unset, None, bool]):
        multipart_data (ImportCsvEmployeeMultipartData):

    Returns:
        Response[Union[Any, List[Item]]]
    """

    return sync_detailed(
        employer_id=employer_id,
        client=client,
        multipart_data=multipart_data,
        format_=format_,
        preview_only=preview_only,
        allow_updates=allow_updates,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    *,
    client: Client,
    multipart_data: ImportCsvEmployeeMultipartData,
    format_: Union[Unset, None, CsvFileFormat] = UNSET,
    preview_only: Union[Unset, None, bool] = False,
    allow_updates: Union[Unset, None, bool] = False,
) -> Response[Union[Any, List[Item]]]:
    """Import CSV

     Import employee data from a CSV file.

    Args:
        employer_id (str):
        format_ (Union[Unset, None, CsvFileFormat]):
        preview_only (Union[Unset, None, bool]):
        allow_updates (Union[Unset, None, bool]):
        multipart_data (ImportCsvEmployeeMultipartData):

    Returns:
        Response[Union[Any, List[Item]]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        multipart_data=multipart_data,
        format_=format_,
        preview_only=preview_only,
        allow_updates=allow_updates,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    *,
    client: Client,
    multipart_data: ImportCsvEmployeeMultipartData,
    format_: Union[Unset, None, CsvFileFormat] = UNSET,
    preview_only: Union[Unset, None, bool] = False,
    allow_updates: Union[Unset, None, bool] = False,
) -> Optional[Union[Any, List[Item]]]:
    """Import CSV

     Import employee data from a CSV file.

    Args:
        employer_id (str):
        format_ (Union[Unset, None, CsvFileFormat]):
        preview_only (Union[Unset, None, bool]):
        allow_updates (Union[Unset, None, bool]):
        multipart_data (ImportCsvEmployeeMultipartData):

    Returns:
        Response[Union[Any, List[Item]]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            client=client,
            multipart_data=multipart_data,
            format_=format_,
            preview_only=preview_only,
            allow_updates=allow_updates,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.rti_agent import RtiAgent
from ..models.rti_contact import RtiContact
from ..models.rti_sender_type import RtiSenderType
from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiSubmissionSettings")


@attr.s(auto_attribs=True)
class RtiSubmissionSettings:
    """
    Attributes:
        sender_type (Union[Unset, RtiSenderType]):
        sender_id (Union[Unset, None, str]): The SenderId used to submit RTI documents to HMRC
        password (Union[Unset, None, str]): The Password used to submit RTI documents to HMRC
        exclude_nil_paid (Union[Unset, bool]): Whether or not to include Employees paid a zero amount on your FPS
        include_hash_cross_ref (Union[Unset, bool]): [readonly] Whether or not the Bacs Cross Ref field is included on
            your FPS submissions
            This is automatically set to true if you use a bank payments CSV format that supports it
            or set to false if not
        auto_submit_fps (Union[Unset, bool]): If set to true, we'll automatically send your FPS to HMRC whenever you
            finalise a PayRun
        test_in_live (Union[Unset, bool]): Used for testing the RTI gateway. If set to true then the Document Type name
            will have "-TIL" appended to it
        use_test_gateway (Union[Unset, bool]): If set to true then your RTI documents will be sent to HMRCs test
            services instead of the live service
        override_timestamp_value (Union[Unset, None, str]): If a value is provided then it will be used as the timestamp
            on the RTI submission. This would normally only be used for testing purposes.
        contact (Union[Unset, RtiContact]):
        agent (Union[Unset, RtiAgent]):
    """

    sender_type: Union[Unset, RtiSenderType] = UNSET
    sender_id: Union[Unset, None, str] = UNSET
    password: Union[Unset, None, str] = UNSET
    exclude_nil_paid: Union[Unset, bool] = UNSET
    include_hash_cross_ref: Union[Unset, bool] = UNSET
    auto_submit_fps: Union[Unset, bool] = UNSET
    test_in_live: Union[Unset, bool] = UNSET
    use_test_gateway: Union[Unset, bool] = UNSET
    override_timestamp_value: Union[Unset, None, str] = UNSET
    contact: Union[Unset, RtiContact] = UNSET
    agent: Union[Unset, RtiAgent] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        sender_type: Union[Unset, str] = UNSET
        if not isinstance(self.sender_type, Unset):
            sender_type = self.sender_type.value

        sender_id = self.sender_id
        password = self.password
        exclude_nil_paid = self.exclude_nil_paid
        include_hash_cross_ref = self.include_hash_cross_ref
        auto_submit_fps = self.auto_submit_fps
        test_in_live = self.test_in_live
        use_test_gateway = self.use_test_gateway
        override_timestamp_value = self.override_timestamp_value
        contact: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contact, Unset):
            contact = self.contact.to_dict()

        agent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.agent, Unset):
            agent = self.agent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if sender_type is not UNSET:
            field_dict["senderType"] = sender_type
        if sender_id is not UNSET:
            field_dict["senderId"] = sender_id
        if password is not UNSET:
            field_dict["password"] = password
        if exclude_nil_paid is not UNSET:
            field_dict["excludeNilPaid"] = exclude_nil_paid
        if include_hash_cross_ref is not UNSET:
            field_dict["includeHashCrossRef"] = include_hash_cross_ref
        if auto_submit_fps is not UNSET:
            field_dict["autoSubmitFps"] = auto_submit_fps
        if test_in_live is not UNSET:
            field_dict["testInLive"] = test_in_live
        if use_test_gateway is not UNSET:
            field_dict["useTestGateway"] = use_test_gateway
        if override_timestamp_value is not UNSET:
            field_dict["overrideTimestampValue"] = override_timestamp_value
        if contact is not UNSET:
            field_dict["contact"] = contact
        if agent is not UNSET:
            field_dict["agent"] = agent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _sender_type = d.pop("senderType", UNSET)
        sender_type: Union[Unset, RtiSenderType]
        if isinstance(_sender_type, Unset):
            sender_type = UNSET
        else:
            sender_type = RtiSenderType(_sender_type)

        sender_id = d.pop("senderId", UNSET)

        password = d.pop("password", UNSET)

        exclude_nil_paid = d.pop("excludeNilPaid", UNSET)

        include_hash_cross_ref = d.pop("includeHashCrossRef", UNSET)

        auto_submit_fps = d.pop("autoSubmitFps", UNSET)

        test_in_live = d.pop("testInLive", UNSET)

        use_test_gateway = d.pop("useTestGateway", UNSET)

        override_timestamp_value = d.pop("overrideTimestampValue", UNSET)

        _contact = d.pop("contact", UNSET)
        contact: Union[Unset, RtiContact]
        if isinstance(_contact, Unset):
            contact = UNSET
        else:
            contact = RtiContact.from_dict(_contact)

        _agent = d.pop("agent", UNSET)
        agent: Union[Unset, RtiAgent]
        if isinstance(_agent, Unset):
            agent = UNSET
        else:
            agent = RtiAgent.from_dict(_agent)

        rti_submission_settings = cls(
            sender_type=sender_type,
            sender_id=sender_id,
            password=password,
            exclude_nil_paid=exclude_nil_paid,
            include_hash_cross_ref=include_hash_cross_ref,
            auto_submit_fps=auto_submit_fps,
            test_in_live=test_in_live,
            use_test_gateway=use_test_gateway,
            override_timestamp_value=override_timestamp_value,
            contact=contact,
            agent=agent,
        )

        return rti_submission_settings
